/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ***** GA: Provides a superclass for basic Genetic Algorithm routines ***** */
/* ************************************************************************** */
abstract class GA
{
	protected int	d_chrom=0;	// last random chromosome selection
	protected int	d_gene=0;	// last random gene selection 


	/** abstract classes to be implemented for specific subclasses */ 
	/** determine fitness of a given GA imlementation */
	public abstract boolean isFit ();
	public abstract float fitness ();

	/** select a random chromosome
	public abstract int rndChrom ();
	public abstract int rndChrom (int offset);
	/** select a random gene
	public abstract int rndGene ();
	public abstract int rndGene (int chromosome, boolean cross_before_number);



	/** Crossover: Does a random crossover of p1 and p2 (parents) */
	public static String[] crossover (String p1, String p2)
		{
		return (crossover (p1, p2, -1));
		}

	/** Crossover: Does a crossover of p1 and p2 (parents) at gene n, 
		returning an array of 2 Children
	**/
	public static String[] crossover (String p1, String p2, int n)
		{
		String	Children[] = new String[2];

		if (n == -1)
			if (p1.length() > p2.length())
				n = (int)(JAC_GlobalRandom.nextFloat()*p2.length());
			else
				n = (int)(JAC_GlobalRandom.nextFloat()*p1.length());

		if (JAC_GlobalDebug.debug() == 3)
			System.out.println ("Crossover: " + p1 + " + " + p2 + 
						" at " + n);

		Children[0] = new String (p1.substring(0,n) + p2.substring (n,p2.length()));
		Children[1] = new String (p2.substring(0,n) + p1.substring (n,p1.length()));

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("Result: " + Children[0] + 
						" + " + Children[1]);

		return (Children);
		}

	/** Crossover: Does a crossover of p1 at gene n1 and p2 at gene n2 
		returning an array of 2 Children
	**/
	public static String[] crossover ( String p1, String p2, int n1, int n2)
		{
		String	Children[] = new String[2];	// array of child chromosomes

		if (n1 == -1)
			n1 = (int)(JAC_GlobalRandom.nextFloat()*p1.length());
		if (n2 == -1)
			n2 = (int)(JAC_GlobalRandom.nextFloat()*p2.length());

		if (JAC_GlobalDebug.debug() == 3)
			System.out.println ("Crossover: " + p1 + " + " + p2 + 
						" at " + n1 + ", " + n2);

		Children[0] = new String (p1.substring(0,n1) + p2.substring (n2,p2.length()));
		Children[1] = new String (p2.substring(0,n2) + p1.substring (n1,p1.length()));

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("Result: " + Children[0] + 
						" + " + Children[1]);

		return (Children);
		}


	/** BreedLottery: Accept a Vector of EPop Vector subscripts (ie: the 
		vector indexes of the selected organisms) and returns the 
		breeding schema for a given population as a vector of Integers.  
		If nslots is odd, a breedingplan for nslots+1 indexes is 
		returned.
	**/
	public static Vector breedLottery (Vector EPop, int nslots)	// ElegiblePopulation index, available slots
		{
		int	psize = EPop.size();
		long	c=0;
		Vector 	VBreed = new Vector (nslots, nslots);// vector of integers, the breed plan

		if (JAC_GlobalDebug.debug() > 0)
			System.out.print ("BreedLottery " + psize + " [" + nslots + "]: ");

		//for (int i=0; i<EPop.size(); i++)
                //        System.out.println ((Integer)EPop.elementAt(i));

		for (int i=0; i<nslots; i+=2)
			{
			int		r1, r2;
			Integer		Winner1, Winner2;

			r1 = (int)(JAC_GlobalRandom.nextFloat()*psize);
			if (r1 == psize)
				r1--;
			Winner1 = (Integer)EPop.elementAt(r1);
			VBreed.addElement (Winner1);
			do 
				{
				r2 = (int)(JAC_GlobalRandom.nextFloat()*psize);
				if (r2 == psize)
					r2--;
				Winner2 = (Integer)EPop.elementAt(r2);
				System.out.println (Winner1 + ", " + Winner2);
				if (++c > psize*50)
					{
					System.out.println ("c > " + psize*50 + " ... infinite loop autodetect ...");
					System.exit (1);
					}
					
				}
			while (Winner2.equals(Winner1));
			VBreed.addElement (Winner2);

			if (JAC_GlobalDebug.debug() > 0)
				System.out.print (Winner1 + "+" + Winner2 + " ");
			}

		if (JAC_GlobalDebug.debug() > 0)
				System.out.println ();

		return (VBreed);
		}
}


