/* ************************************************************************* *
   JAC - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	     http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ***** JAC_FileReader: extends FileReader, adds a few utility functions *** */
/* ************************************************************************** */
public class JAC_FileReader extends BufferedReader
{
	public final int	MAXSTRINGSIZE=255;

	private int		d_maxsize=MAXSTRINGSIZE;
	private boolean		d_ignore_blanks=false;

	/** Constructor: call FileReader constructor with filename */
	public JAC_FileReader (String fname) throws FileNotFoundException 
		{
		super (new FileReader (fname));
		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("Reading File " + fname);
		}

	/** Constructor: set d_ignore_blanks mode, call other constructor */
	public JAC_FileReader (String fname, boolean ignore_blanks) 
			throws FileNotFoundException 
		{
		this (fname);
		d_ignore_blanks=ignore_blanks;
		}

	/** Constructor: set d_ignore_blanks mode, call other constructor */
	public JAC_FileReader (String Dir, String Fle) throws FileNotFoundException 
		{
		super (new FileReader (new File (Dir, Fle)));
		if (JAC_GlobalDebug.debug() > 3)
			System.out.println ("Reading File " + Dir + "/" + Fle);
		}

	/** finalize: have destructor close file */
	protected void finalize() throws Throwable 
		{
		this.close ();
		}

	/** readList: default call, read until EOF */
	public Vector readList ()
		{
		return (readList(0, null, null));
		}

	/** readList: read a list (1 entry per line) with a maximum 
	    of max entries (max==0 will read the entire list) returning
	    either at EOF or when a line starts with the string stop
	**/
	public Vector readList (int max, String StopAt, String Comment)
		{
		Vector		VList=new Vector (5, 5);
		String		Line=null;

		if (StopAt == null)
			StopAt = new String();

		try {
		    while ((Line=readLine())!=null)
			{
			Line = Line.trim();
			if (Line.length() > 0 && !Line.startsWith(Comment))
				{
				int	c, i;

				c = Line.indexOf ("#");
				if (c != -1)
					Line = new String (Line.substring (0, c));
				VList.addElement (Line);
				if (Line.startsWith(StopAt))
					return (VList);
				if (max>0  && VList.size() == max)
					break;
				}
			}
		     this.close();
		
		    }
		catch (IOException e)
		    {
		    System.out.println ("Caught IOException in JAC_FileReader::readList () ...");
		    return (null);
		    }
			
		return (VList);
		}

	/** readBuffer: read and return a buffer no greater than max chars */
	public char[] readBuffer (int	max) 
		{
		char[]		buf = new char[max];

		try {
			read (buf, 0, max);
			return (buf);
		    }
		catch (IOException e)
		    {
		    System.out.println ("Caught IOException in JAC_FileReader::readBuffer() ...");
		    return (null);
		    }
		}
}

