/* ************************************************************************* *
   JAC - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	     http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* *** JAC_FileUtil: extends File, adds utility methods (some are static) *** */
/* ************************************************************************** */
public class JAC_FileUtil extends File
{
	public final int	MAXSTRINGSIZE=255;
	public String 		d_Fname=null;
	public String		d_Dir=null;

	private int		d_maxsize=MAXSTRINGSIZE;
	private boolean		d_ignore_blanks=false;

	/** Constructor: call assign the filname to use */
	public JAC_FileUtil (String Fname) throws FileNotFoundException 
		{
		super (Fname);

		d_Fname=Fname;
		}

	/** Constructor: call assign the filname to use */
	public JAC_FileUtil (String Dir, String Fname) throws FileNotFoundException 
		{
		super (new File(Dir), Fname);

		d_Dir=Dir;
		d_Fname=Fname;
		}

	/** BaseName: return the base of the filename (doing a `cut -f1 -d.`) */
	public static String baseName (String Fname)
		{
		int		ds=0, last=0,
				lim=Fname.length();
		String		Basename=null;

		if ((ds=Fname.lastIndexOf("/")) != -1)
			Fname=Fname.substring(ds, lim);

		Basename = new String (Fname);
		if (Basename.lastIndexOf('.') != -1)
			return (Basename.substring (0, Basename.lastIndexOf('.')));
		else
			return (Basename);
		}

	/** getPwd: return the current working directory */
	public static String getPwd ()
		{
		return (System.getProperties().getProperty("user.dir"));
		}

	/** ChangeDir: change the current working directory */
	public static void setPwd (String Dir)
		{
		Properties 	P=System.getProperties();

		P.put("user.dir", Dir);
		System.setProperties(P);
		}

	/** moveTo: return 'this' file to directory Dest */
	public boolean moveTo (String Dest) 
		{
		boolean rc=false;
		File 	D = new File (Dest, d_Fname);

		if (!new File(Dest).isDirectory())
			{
			System.out.println ("JAC_FileUtil::moveTo(): " + Dest + " is not a directory");
			return (false);
			}

		if (this.exists())
			rc = this.renameTo (D);
		//System.out.println ("moveTo: " + this + " " + D);
		return (rc);
		}

	/** isNewer: see if 'this' file is newer than Fname2 */
	public boolean isNewer (String Fname2) throws IOException, 
						      FileNotFoundException
		{
		File	F2 = new File (Fname2);

		if (!F2.exists())
			return (false);

		if (this.lastModified() > F2.lastModified())
			return (true);
		else
			return (false);
		}

	/** copy: copy d_Fname to FOutName */
	public int copy (String FOutName)
		{
		FileInputStream		Fin=null; 
		FileOutputStream	Fout=null;
		byte[]			buf = new byte[512];
		int			n;

		try {
		    Fin = new FileInputStream (this);
		    Fout = new FileOutputStream (FOutName);

		    while (true)
			{
			n=Fin.read (buf, 0, 512);
			if (n == -1)
				break;
			else
				Fout.write (buf, 0, n);
			}
		    Fin.close();
		    Fout.close();
		    }
		catch (Exception e) {
		    System.out.println ("Caught Exception in JAC_FileUtil::copy: " + e);
		    return (-1);
		    }

		return (0);
		}

	/** stringReplace: do a string replace on a file, creating 
	    NewFname in the process. 
	**/
	public int stringReplace (String OrgStr, String NewStr)
				throws FileNotFoundException 
		{
		return (stringReplace (OrgStr, NewStr, null));
		}


	/** stringReplace: do a string replace on a file, overwriting the 
	    the original file 
	**/
	public int stringReplace (String OrgStr, String NewStr, String NewFname)
				throws FileNotFoundException 
		{
		Vector 	V1 = new Vector (1, 1),
			V2 = new Vector (1, 1);

		V1.addElement (OrgStr);
		V2.addElement (NewStr);
		return (stringReplace (V1, V2, NewFname));
		}

	/** stringReplace: do a vectorized string replace on a file, 
	    overwriting the the original file 
	**/
	public int stringReplace (Vector VOrgStr, Vector VNewStr)
				throws FileNotFoundException 
		{
		return (stringReplace (VOrgStr, VNewStr, null));
		}

	/** stringReplace: do a vectorized string replace on a file, creating 
	    NewFname in the process. If NewFname==null, the original file 
	    is replaced.
	**/
	public int stringReplace (Vector VOrgStr, Vector VNewStr, String NewFname) 
				throws FileNotFoundException 
		{
		int		lim = VOrgStr.size();
		File		Of=null, Nf=null;	// old file, new file
		JAC_FileReader	Rd=null;
		FileWriter	Wr=null;
		String		Line=null, 
				NewLine=null,
				Outname=null,
				OrgStr=null,
				NewStr=null;

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("stringReplace on File " + d_Fname);

		if (lim != VNewStr.size())
			{
			System.out.println ("Src and New vectors differ in size ...");
			return (1);
			}

		// initialize Reader
		if (d_Dir == null)
			Rd = new JAC_FileReader (d_Fname);
		else
			Rd = new JAC_FileReader (d_Dir, d_Fname);

		// do we replace the original file?
		if (NewFname == null)
			Outname = new String ("." + d_Fname);
		else
			Outname = NewFname;

		// create temporary file
		try {
			Wr = new FileWriter (Outname);

			while ((Line=Rd.readLine()) != null)
				{
				for (int i=0; i<lim; i++)
					{
					OrgStr=(String)VOrgStr.elementAt (i);
					NewStr=(String)VNewStr.elementAt (i);
					Line = JAC_StringUtil.stringReplace(Line,OrgStr,NewStr);
					}
				Wr.write (Line, 0, Line.length());
				Wr.write ("\n");
				}

			Wr.close ();
			Rd.close ();

			// finalize files 
			if (NewFname == null)
				{
				// delete original file
				this.delete();

				// rename copy to original name 
				if (d_Dir != null)
					Nf = new File (new File (d_Dir), Outname);
				else
					Nf = new File (Outname);
				Nf.renameTo (this);
				}
			}
		catch (IOException e)
			{
			System.out.println ("Caught IOException in JAC_FileUtil::stringReplace \n");
			return (-1);
			}

		return (0);
		}
}
