/* ************************************************************************* *
   JAC - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	     http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ****** JAC_GlobalRandom: Provides a static (global) Random mechanism ***** */
/* ************************************************************************** */
class JAC_GlobalRandom 
{
	private static Random		d_Rnd=null;

	/** Init: create a default random number generator */
	public static void init ()
		{
		if (d_Rnd == null)
			{
			long		seed;
			Date		D = new Date ();

			seed = D.getTime ();
			d_Rnd = new Random (seed);
			if (JAC_GlobalDebug.debug() > 0)
				System.out.println ("Initialized JAC_GlobalRandom() with seed = " + seed);
			}
		else
			if (JAC_GlobalDebug.verbose())
				System.out.println ("Warning: JAC_GlobalRandom can only be initialized once ... ignored init");
		}

	/** Init: create a random number generator with specified seed */
	public static void init (long seed)
		{
		if (d_Rnd == null)
			{
			d_Rnd = new Random (seed);
			if (JAC_GlobalDebug.debug() > 0)
				System.out.println ("Initialized JAC_GlobalRandom() with seed = " + seed);
			}
		else
			if (JAC_GlobalDebug.verbose())
				System.out.println ("Warning: JAC_GlobalRandom can only be initialized once ... ignored init");
		}

	/** Init: override nextFloat() method */
	public static float nextFloat ()
		{
		verify_init (d_Rnd);
		return (d_Rnd.nextFloat());
		}

	/** Init: override nextDouble() method */
	public static double nextDouble()
		{
		verify_init (d_Rnd);
		return (d_Rnd.nextDouble());
		}

	/** verify_init: make sure Rnd is initialized */
	private static void verify_init (Random Rnd)
		{
		if (Rnd == null)
			{
			System.out.println ("Forcing Init() on uninitialized JAC_GlobalRandom ...");
			init (new Date().getTime());
			}
		}
}

