/* ************************************************************************* *
   JAC - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	     http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.awt.*;


/* ************************************************************************* */
/* ************* JAC_ImageButton: create a button with an image ************ */
/* ************************************************************************* */
public class JAC_ImageButton extends Component {
	
	private Image d_Image;

	/** Constructor: default */
	public JAC_ImageButton() 
		{
		this(null); 
		}
	
	/** Constructor: specify an Image */
	public JAC_ImageButton(Image image) 
		{
		d_Image=image;
		}
	
	/** getImage: return the current image */
	public Image getImage() 
		{
		return (d_Image);
		}
	
	/** setImage: set the image if it is different from the current one */
	public void setImage(Image image) 
		{
		if(!image.equals(d_Image)) 
			{
			d_Image = image;
			repaint();
			}
		}
	
	/** paint: repaint the button */
	public void paint(Graphics g) 
		{
		Color b = getBackground();
		Font f = g.getFont();
		Rectangle r = g.getClipBounds();
		
		super.paint(g);
		
		// Draw the image.
		// Scale it so it fits in the clip rectangle and centre it vertically.
		if(d_Image != null && r.width>4 && r.height>4) 
			{
			Dimension d = new Dimension(d_Image.getWidth(this), d_Image.getHeight(this));
			float scale=1, scalew=1, scaleh=1;
			float fdwidth = (new Integer(d.width)).floatValue();
			float frwidth = (new Integer(r.width)).floatValue();
			float fdheight = (new Integer(d.height)).floatValue();
			float frheight = (new Integer(r.height)).floatValue();
			if(fdwidth > frwidth || fdheight > frheight)
				{
				scalew = 1 * (frwidth/fdwidth);
				scaleh = 1 * (frheight/fdheight);
				if (scalew < scaleh)
					scale = scalew;
				else
					scale = scaleh;
				}
				
			int ndwidth = (new Float(fdwidth * scale)).intValue() - 4;
			int ndheight = (new Float(fdheight * scale)).intValue() - 4;
			int xOffset = (r.width - ndwidth) / 2;
			int yOffset = (r.height - ndheight) / 2;
			g.drawImage(d_Image, xOffset, yOffset, ndwidth, ndheight, this);
			}
		
		// Draw the lines twice to make them thicker.
		g.setColor (b.brighter());
		g.drawLine (0, r.height-1, 0, 0);
		g.drawLine (1, r.height-2, 1, 1);
		g.drawLine (0, 0, r.width-1, 0);
		g.drawLine (1, 1, r.width-2, 1);
		g.setColor (b.darker());
		g.drawLine (r.width-1, 0, r.width-1, r.height-1);
		g.drawLine (r.width-2, 1, r.width-2, r.height-2);
		g.drawLine (r.width-1, r.height-1, 0, r.height-1);
		g.drawLine (r.width-2, r.height-2, 1, r.height-2);
	}
}
