/* ************************************************************************* *
   JAC - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	     http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.io.*;
import java.awt.*;
import java.awt.event.*;


/* ************************************************************************** */
/* ** JAC_ImageTextButton: component consisting of Button, Image and Label ** */
/* ************************************************************************** */
public class JAC_ImageTextButton extends Component 
{
	private String 	d_Label=null;			// the text label
	private Image 	d_Image=null;			// the image 
	private	boolean	d_show_label=true;		// do we show the label
	private	boolean	d_keep_aspect=false;		// do we keep the source aspect?

	/** Constructor: create the ITB, set Label and Button image */
	public JAC_ImageTextButton(String Label, String ImgFile) 
		{
		File	F;

		if (Label == null)
			d_show_label = false;
		else
			d_Label = Label;

		setImage (ImgFile);	
		}

	/** Constructor: create the ITB, set Label and Button image */
	public JAC_ImageTextButton(String Label, Image image) 
		{
		if (Label == null)
			d_show_label = false;
		else
			d_Label = Label;
		d_Image = image;
		}

	public void windowActivated (WindowEvent e)
		{
		this.repaint();
		}

	/** getKeepAspect: return the keep_aspect setting */
	public boolean getKeepAspect ()
		{
		return (d_keep_aspect);
		}

	/** setShowLabel: set the show label mode */
	public void setShowLabel (boolean show_label)
		{
		if (show_label != d_show_label)
			{
			d_show_label = show_label;
			repaint();
			}
		}

	/** setKeepAspect: set the keep_aspect flag */
	public void setKeepAspect (boolean keep_aspect)
		{
		if (keep_aspect != d_keep_aspect)
			{
			d_keep_aspect = keep_aspect;
			repaint();
			}
		}

	/** getLabel: return the Label text */
	public String getLabel() 
		{
		return (d_Label);
		}
	
	/** getImage: return the Image */
	public Image getImage() 
		{
		return (d_Image);
		}
	
	/** setLabel: set the Label */
	public void setLabel(String Label) 
		{
		if (Label == null)
			d_show_label = false;
		else
			{
			d_Label = Label;
			d_show_label = true;
			}
		repaint();
		}
	
	/** setLabel: set the Image */
	public void setImage(String ImgFile) 
		{
		File	F;

		F = new File (ImgFile);	
		if (F.exists())
			d_Image = Toolkit.getDefaultToolkit().getImage (ImgFile);
		repaint();
		}

	/** setLabel: set the Image */
	public void setImage(Image image) 
		{
		if (image != null)
			{
			if(!image.equals(d_Image)) 
				d_Image = image;
			}
		else
			d_Image = image;
		repaint();
		}

	/** Remaining, non-implemented methods in MouseListener */
        public void mouseEntered (MouseEvent e)         { }
        public void mouseExited (MouseEvent e)          { }
        public void mousePressed (MouseEvent e)         { } 
        public void mouseReleased (MouseEvent e)        { }


	/** paint: repaint the button */
	public void paint(Graphics g) 
		{
		Color 		b = getBackground();
		Font 		f = g.getFont();
		Rectangle 	r = g.getClipBounds();
		FontMetrics 	fm = g.getFontMetrics();
		int		ndwidth, ndheight, 
				xOffset, yOffset;

		super.paint(g);
		
		// Draw the image.
		// Scale it so it fits in the clip rectangle and centre it vertically.
		if(d_Image != null && r.width>4 && r.height>4) 
			{
			Dimension d = new Dimension(d_Image.getWidth(this), d_Image.getHeight(this));
			float scale=1, scalew=1, scaleh=1;
			float fdwidth = (new Integer(d.width)).floatValue();
			float frwidth = (new Integer(r.width)).floatValue();
			float fdheight = (new Integer(d.height)).floatValue();
			float frheight = (new Integer(r.height)).floatValue();
			if(fdwidth > frwidth || fdheight > frheight)
				{
				scalew = 1 * (frwidth/fdwidth);
				scaleh = 1 * (frheight/fdheight);
				if (scalew < scaleh)
					scale = scalew;
				else
					scale = scaleh;
				}

			if (d_keep_aspect)
				{
				ndwidth = (new Float(fdwidth * scale)).intValue() - 4;
				ndheight = (new Float(fdheight * scale)).intValue() - 4;
				xOffset = (r.width - ndwidth) / 2;
				yOffset = (r.height - ndheight) / 2;
				}
			else
				{
				ndwidth = (new Float(fdwidth * scalew)).intValue(); 
				ndheight = (new Float(fdheight * scaleh)).intValue();
				xOffset = 0;
				yOffset = 0;
				}

			if(d_show_label)
				ndheight = ndheight-(int)(fm.getAscent()*1.5);
			else
				yOffset = (int)((r.height - ndheight) / 2); 

			// System.out.println (fdwidth + " " + fdheight + ", " + frwidth + " " + frheight + ", " + yOffset + " " + ndwidth + " " +  ndheight); 

			g.drawImage(d_Image, xOffset, yOffset, ndwidth, ndheight, this);
			}
		
		// Draw the label.
		// Centre it in the clip rectangle.
		if(d_show_label)
			g.drawString(d_Label, (r.width - fm.stringWidth(d_Label)) / 2, r.height - (fm.getAscent()/2));
		
		// Draw the lines twice to make them thicker.
		g.setColor (b.brighter());
		g.drawLine (0, r.height-1, 0, 0);
		g.drawLine (1, r.height-2, 1, 1);
		g.drawLine (0, 0, r.width-1, 0);
		g.drawLine (1, 1, r.width-2, 1);
		g.setColor (b.darker());
		g.drawLine (r.width-1, 0, r.width-1, r.height-1);
		g.drawLine (r.width-2, 1, r.width-2, r.height-2);
		g.drawLine (r.width-1, r.height-1, 0, r.height-1);
		g.drawLine (r.width-2, r.height-2, 1, r.height-2);
		}
}
