/* ************************************************************************* *
   JAC - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	     http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.awt.*;
import java.awt.Frame;
import java.awt.event.*;
import java.awt.MenuShortcut;
import java.awt.event.ActionListener;
import java.io.*;
import java.util.Properties;
import java.awt.datatransfer.*;


/* *********************************************************************** */
/* ********** JAC_SimpleTextEdit: provide a simple text editor *********** */
/* *********************************************************************** */
public class JAC_SimpleTextEdit extends Thread implements ActionListener 
{
	protected TextArea 	d_Ta;
	protected Frame		d_Frame;
	protected String 	d_Name;
	protected boolean	d_charbuf = false; 	// indicates that we're talking from object to object

	private JAC_DialogYN 	d_DlgYN;
	private JAC_DialogOK 	d_DlgOK;
	//private Label 		d_InfoLabel;
	private static int 	d_offx = 0;
	private static int 	d_offy = 0;


	/** main: run standalone */
	public static void main(String args[])
		{
		JAC_SimpleTextEdit STE;

		if (args.length > 0)
			STE = new JAC_SimpleTextEdit (args[0]);
		else
			STE = new JAC_SimpleTextEdit ();
		}

	/** Constructor: bring up an empty text editor */
	public JAC_SimpleTextEdit ()
		{
		super ();
		d_Name = new String ("Untitled.txt");
		init ();
		}

	/** Constructor: edit a file in the text editor */
	public JAC_SimpleTextEdit (String Fname)
		{
		super ();
		init ();
		readFile (null, Fname);
		d_Name = new String (Fname);
		}

	/** run: thread runner */
	public void run ()
		{
		d_Frame.show ();
		}

	/** close: close the text editor and return it's contents */
	public String close ()
		{
		d_Frame.dispose ();
		return (d_Ta.getText ());
		}

	/** init: initialize the gui */
	public void init ()
		{
		d_Frame = new Frame ();
		d_Ta = new TextArea();

		d_Frame.setLayout (new BorderLayout()); 
		//d_Frame.add (d_InfoLabel = new Label ("", FlowLayout.CENTER), "South");
		//d_InfoLabel.setText ("Test");
		d_Frame.add ("Center", d_Ta);
		d_Frame.setSize (500, 500);

		d_offx += 50;
		d_offy += 50;
		d_Frame.setLocation (d_offx, d_offy);

		MenuBar myMenuBar = new MenuBar();

		makeFileMenu (myMenuBar);
		d_Frame.setResizable (true);
		d_Frame.setTitle ("SimpleTextEdit");
		d_Frame.setMenuBar (myMenuBar);
		}

	/** addItem: add a menu item with a shortcut */
	public void addItem (Menu Mnu, String S, MenuShortcut MShortcut)
		{
		MenuItem Mi = new MenuItem (S, MShortcut);

		Mi.addActionListener (this);
		Mnu.add (Mi);
		}

	/** addItem: add a menu item without a shortcut */
	public void addItem (Menu Mnu, String S)
		{
		MenuItem Mi = new MenuItem (S);

		Mi.addActionListener(this);
		Mnu.add(Mi);
		}

	/** makeFileMenu: construct the file menu */
	public void makeFileMenu (MenuBar Mbar) 
		{
		Menu fileMenu = new Menu("File");

		addItem(fileMenu, "New");
		addItem(fileMenu, "Open");
		addItem(fileMenu, "Close");
		fileMenu.addSeparator();
		if (!d_charbuf)
			addItem(fileMenu, "Save", new MenuShortcut(KeyEvent.VK_S));
		addItem(fileMenu, "Save As");

		fileMenu.addSeparator();
		addItem(fileMenu, "Quit");
		
		Mbar.add(fileMenu);
		}

	/** openDialog: construct the open dialog */
	public void openDialog () 
		{
		int 		filesize;
		char[] 		data;
		String 		FinalText = new String(""),
				Fname, Dname;
		FileDialog 	fd = new FileDialog(d_Frame, "Please choose the file to open:", FileDialog.LOAD);

		fd.setFile("*.txt");
		fd.show();
		Fname = fd.getFile();
		Dname = fd.getDirectory ();

		if (Fname!=null && Dname!=null)
			readFile(fd.getDirectory(), fd.getFile());
		}

	/** readFile: read the specified file into the text editor */
	public void readFile (String Pathname, String Filename) 
		{
		try {
			String 		Fname;
			if (Pathname != null)
				Fname = new String (Pathname + Filename);
			else
				Fname = new String (Filename);
			JAC_FileReader 	Jfr = new JAC_FileReader (Fname);
			int 		lim = 10000;
			char[] 		buf = null; 

			buf = Jfr.readBuffer (lim);
			Jfr.close();
			d_Ta.setText (new String (buf, 0, lim));
			d_Ta.select (0,0);

			d_Frame.setTitle ("SimpleTextEdit: " + Filename);
			d_Frame.show ();
		    } // end try
		catch (FileNotFoundException e) {
				System.err.println ("Error: " + e);
			}
		catch (IOException e) {
				System.err.println ("Error: " + e);
			}
		}
	
	/** saveasDialog: display the saveAs dialog and save the file */
	public void saveasDialog () 
		{
		String FieldText, Fname, Dname;
		
		try {
			FileDialog fd = new FileDialog(d_Frame, "Please choose filename to save as:", FileDialog.SAVE);
			fd.setFile(d_Name);
			fd.show();
			Fname = fd.getFile();
			Dname = fd.getDirectory ();

			if (Fname!=null && Dname!=null)
				{
				readFile(fd.getDirectory(), fd.getFile());
				JAC_FileWriter bw = new JAC_FileWriter (fd.getDirectory()+fd.getFile());

				FieldText = d_Ta.getText();
				bw.flush();
				bw.write(FieldText, 0, FieldText.length());
				bw.close();
				d_Frame.setTitle(new String(fd.getDirectory() + fd.getFile()));
				}
			}
		catch (IOException e) {
			System.err.println(e);
			}
		catch (NullPointerException e) {
			System.err.println("Error: " + e);
			}
		}

	/** closeDialog: make sure the user wants to close */
	public void closeDialog() 
		{
		d_DlgYN = new JAC_DialogYN (this, "There are unsaved changes. Are you sure you want to close?");
		d_DlgYN.show ();
		d_DlgYN.dispose ();
		}

	/** closeDialogAction: process any action from the closeDialog */
	public void closeDialogAction (String s) 
		{
		d_DlgYN.setVisible(false);
		if (s.equals("Yes")) 
			d_Ta.setText (new String ());
		else if (s.equals("No")) 
			;
		else 
			System.out.println("Error with closeDialogAction");
		}

	/** aboutDialog: show the about dialog */
	public void aboutDialog () 
		{
		d_DlgOK = new JAC_DialogOK (this, "test");
		d_DlgOK.show ();
		d_DlgOK.dispose ();
		}

	/** aboutDialogAction: process any action from the aboutDialog */
	public void aboutDialogAction (String s) 
		{
		//d_DlgOK.setVisible(false);
		}

	/** actionPerformed: process user interaction from the text editor */
	public void actionPerformed(ActionEvent e) 
		{  
		Runtime rt = Runtime.getRuntime();

		if ((e.getActionCommand()).equals("New"))
			; //JAC_SimpleTextEdit te = new JAC_SimpleTextEdit ();
		else if ((e.getActionCommand()).equals("Open")) 
			openDialog();
		else if ((e.getActionCommand()).equals("Save As")) 
			saveasDialog();
		else if ((e.getActionCommand()).equals("Close"))
			closeDialog();
		else if ((e.getActionCommand()).equals("Quit"))
			close ();
		else if ((e.getActionCommand()).equals("Yes")) 
			closeDialogAction("Yes");
		else if ((e.getActionCommand()).equals("No")) 
			closeDialogAction("No");
		else if ((e.getActionCommand()).equals("Cancel")) 
			closeDialogAction("Cancel");
		else if ((e.getActionCommand()).equals("OK")) 
			aboutDialogAction("OK");
		//else
		//	d_InfoLabel.setText(e.getActionCommand() + " is disabled");    
		}
}

