/* ************************************************************************* *
   JAC - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	     http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ******* JAC_StringUtil: provide a few (static) String util methods ******* */
/* ************************************************************************** */
public class JAC_StringUtil 
{

	/** pReplace: return the new String resulting from replacing 
	    the char needle with newneedle. Each occurrance of needle 
	    has probability p of being replaced.
	**/
	public static String pStringReplace (String Haystack, char needle, 
					char newneedle, float p)
		{
		int	n, last=0;
		String	H=Haystack;

		if (Haystack == null || needle == '\0')
			return (null);

		if (p<0 || p>1)
			JAC_Bailout.bailout ("Sanity Check Failed in JAC_StringUtil::pStringReplace: !(0<=p<=1) ... Exiting");

		do { 
			if ((n=H.substring(last).indexOf ((int)needle)) == -1)
				return (H);
			else
			if (p >= JAC_GlobalRandom.nextFloat())
				H = new String (H.substring(0,last+n) + newneedle + 
					H.substring(last+n+1));

			last += n+1;
		    }
		while (true);
		}

	/** stringReplace: return the new String resulting from replacing 
	    the substring Needle with Newneedle
	**/
	public static String stringReplace (String Haystack, String Needle, 
					    String NewNeedle)
		{
		int	n;
		String	H=Haystack;

		if (Haystack == null || Needle == null)
			return (null);

		do { 
			if ((n=H.indexOf (Needle)) == -1)
				return (H);
			else
				H = new String (H.substring(0,n) + NewNeedle + 
					H.substring(n+Needle.length(), H.length()));
		    }
		while (true);
		}

	/** stringReplace: return the new String resulting from replacing 
	    the substring Needle with Newneedle. Each occurrance of Needle 
	    has probability p of being replaced.
	**/
	public static String pStringReplace (String Haystack, String Needle, 
					    String NewNeedle, float p)
		{
		int	n, last=0;
		String	H=Haystack;

		if (Haystack == null || Needle == null)
			return (null);

		if (p<0 || p>1)
			JAC_Bailout.bailout ("Sanity Check Failed in JAC_StringUtil::pStringReplace: !(0<=p<=1) ... Exiting");

		do { 
			if ((n=H.indexOf(Needle)) == -1)
				return (H);
			else
			if (p >= JAC_GlobalRandom.nextFloat())
				H = new String (H.substring(0,last+n) + NewNeedle + 
					H.substring(last+n+1));

			last += n+1;
		    }
		while (true);
		}

	/** vectorizeString: Return a Vector of Strings consisting 
	    of the Line tokenized accoring to Delim 
	**/
	public static Vector vectorizeString (String Line, String Delim)
		{
		if (Line == null || Delim == null)
			return (null);

		String		Tok;
		Vector		VLine = new Vector (5, 5);
		StringTokenizer	ST = new StringTokenizer (Line, Delim);

		while (ST.hasMoreTokens())
			{
			Tok = ST.nextToken();
			VLine.addElement (Tok);
			}

		return (VLine);
		}

	/** vectorizeString: store the tokens resulting from tokenizing 
	    Line with delimeter Delim in the Vector Vline. 
	**/
	public static Vector vectorizeString (Vector VLine, String Line, 
						String Delim)
		{
		if (VLine == null || Line == null || Delim == null)
			return (null);

		String		Tok;
		StringTokenizer	ST = new StringTokenizer (Line, Delim);

		while (ST.hasMoreTokens())
			{
			Tok = ST.nextToken();
			VLine.addElement (Tok);
			}

		return (VLine);
		}

	/** stringBalance: get the balance of the copen and close characters. 
	    Useful for matching/counting brackets.
	**/
	public static int stringBalance (String Line, char copen, char cclose)
		{
		int	c=0, lim=Line.length();
		char[]	s=Line.toCharArray ();

		for (int i=0; i<lim; i++)
			{
			if (s[i] == copen)		// push 
				c++;
			else
			if (s[i] == cclose)		// pop
				c--;
			}

		return (c);
		}
}


