/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;


/* *********************************************************************** */
/* *** LPEFileUtil: Threadable class to clean up after rendering a LSO *** */
/* *********************************************************************** */
public class LPEFileUtil extends Thread
{
	private String		d_Dir, 
				d_Basename,
				d_Pwd,
				d_Lsname, 
				d_Incname,
				d_Povname, 
				d_Jpgname, 
				d_Tganame;

	/** LPEFileUtil: Constructor: assign modes and names */
	public LPEFileUtil (String Dir, String Basename)
		{
		if (Basename == null)
			JAC_Bailout.bailout ("Sanity check Failed: Basename==null in LPEFileUtil()... Exiting");

		d_Dir = Dir;
		d_Basename = Basename;

		d_Pwd = System.getProperties().getProperty ("user.dir");
		d_Lsname = new String (d_Basename + ".ls");
		d_Incname= new String (d_Basename + ".inc");
		d_Povname = new String (d_Basename + ".pov");
		d_Jpgname = new String (d_Basename + ".jpg");
		d_Tganame = new String (d_Basename + ".tga");
		}

	/** cleanupFiles: clean up all the remaining files */
	public int cleanupFiles ()
		{
		File		D=new File (d_Dir);
		JAC_FileUtil	F;

		try {
		    if (!D.exists())
			D.mkdir ();
		    else
		    if (!D.isDirectory())
			return (-1);

		    // move files into population directory 
		    if (LPEOptions.s_directory)
			{
			F = new JAC_FileUtil (d_Pwd, d_Lsname);
			if (F.exists())
				F.moveTo (d_Dir);

			F = new JAC_FileUtil (d_Pwd, d_Povname);
			if (F.exists())
				F.moveTo (d_Dir);

			F = new JAC_FileUtil (d_Pwd, d_Jpgname);
			if (F.exists())
				F.moveTo (d_Dir);
			}

		    // see if we need to clean out the *.inc files 
		    if (LPEOptions.s_cleanup)
			{
			File	tF;

			if (LPEOptions.s_renderMode == LPERender.s_RENDER_MODE_MBLOB)
				{
				for (int i=0; i<=LPERender.s_MBLOB_INCLUDE_FILES; i++)
					{
					tF = new File (new String ("output" + i + ".inc"));
					if (tF.exists())
						tF.delete ();
					}
				}
			else
				{
				tF=new File (d_Incname);
				if (tF.exists())
					tF.delete ();
				}

			tF=new File (d_Tganame);
			if (tF.exists())
				tF.delete ();
			}
		    else
		    if (LPEOptions.s_directory)
			{
			F = new JAC_FileUtil (d_Pwd, d_Incname);
			if (F.exists())
				F.moveTo (d_Dir);

			F = new JAC_FileUtil (d_Pwd, d_Tganame);
			if (F.exists())
				F.moveTo (d_Dir);
			}

		    // core dump cleanup code
		    File Ff = new File ("core");
		    if (Ff.exists())
		    	Ff.delete ();
		    }
		catch (Exception e)
			{ System.out.println ("Caught exception in LPEFileUtil::cleanupFiles(): " + e); }
		return (0);
		}
}

