/* ************************************************************************* * 
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;


/* ************************************************************************** */
/* * LPEGui: threaded Frame class: build and run the gui routines/callbacks * */
/* ************************************************************************** */
public class LPEGui extends Thread implements 	ActionListener, 
						MouseListener
						//EventListener
{
	public static final int		s_MAX_FILESIZE = 5*1024*1024;
	public static final int		s_MIN_FILESIZE = 50*1024;
	public static final String 	s_INIT_BTN = new String ("Ready");
	public static final String 	s_TITLE_PREFIX = new String ("LPEvol: ");

	private MenuItem	d_FileNew = new MenuItem ("New");
	private MenuItem	d_FileOpen = new MenuItem ("Open");
	private MenuItem	d_FileSave = new MenuItem ("Save");
	private MenuItem	d_FilePrefs = new MenuItem ("Preferences");
	private MenuItem	d_FileExit = new MenuItem ("Exit");
	private MenuItem	d_LParserBreed = new MenuItem ("Breed");
	private MenuItem	d_AboutAbout = new MenuItem ("About");
	private Vector		d_VButtons = new Vector (5, 5);
	private FileDialog 	d_FD = null;
	private Frame		d_Frame = null;
	private LParserPop	d_LPP = null;
	private String		d_Title = null;
	private	int		d_numButtons = 0;

	// useful for standalone testing 
	// public static void main (String[] args) { LPEGui lsg=new LPEGui ("test"); }  

	/** Constructor: create the LParserEvol gui and start rendering */
	public LPEGui (LParserPop LPP, String Title)
		{
		// initialize Frame object 
		super (Title);

		// sanity check 
		if (LPP == null)
			{
			System.out.println ("Error: _LPP_ == null in LPEGui() ... Exiting\n");
			System.exit (1);
			}

		d_LPP = LPP;
		d_Title = Title;
		if ((LPEOptions.s_maxincfile>0 && 
		     LPEOptions.s_maxincfile<s_MIN_FILESIZE) || 
		     LPEOptions.s_maxincfile<0)
			{
			System.out.println ("Warning: Invalid maxincfile (" + LPEOptions.s_maxincfile + 
					    " ... ignored");
			}
		}

	/** run: thread initialization */
	public void run ()
		{
		int		n=0, lim = d_LPP.size();
		boolean		renderAll=true;

		init ();
		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("Initialized Thread: " + d_Frame.getName());

		d_Frame.show ();

		if (lim > 0 && LPEOptions.s_autostart)
			{
			LParserObj	LPO = (LParserObj)d_LPP.elementAt(0);
			String		Dirname = LPO.generationName(), 
					Basename = LPO.fullName();

			if (LPEOptions.s_recycleImages)
				{
				n = recycleImages ();
				if (n > 0)
					renderAll = false;
				}

			if (n < d_LPP.size())
				{
				d_LPP.write ();
				d_Frame.setTitle (new String (s_TITLE_PREFIX + 
					d_LPP.generationName()));
				if (LPEOptions.s_quit)
					{
					LPEFileUtil	LFU = null;
				
					d_LPP.render (d_VButtons, true, true, renderAll);
					d_Frame.setVisible (false);
					d_Frame.dispose ();
					LFU = new LPEFileUtil (Dirname, Basename);
					LFU.cleanupFiles ();
					System.exit (0);
					}
				else
					d_LPP.render (d_VButtons, false, true, renderAll);
				}
			}
		else
		if (lim > 0 && !LPEOptions.s_autostart)
			if (LPEOptions.s_recycleImages)
				n = recycleImages ();
				
		}

	/** paint: called through repaint() or automatically */
	public void paint (Graphics g)
		{
		int			lim;
		JAC_ImageTextButton	Btn;

		if (d_LPP != null)
			lim = d_LPP.size();
		else
			lim = 0;

		System.out.println ("REPAINTING " + lim);
		for (int i=0; i<lim; i++)
			{
			Btn=(JAC_ImageTextButton)d_VButtons.elementAt(i);
			Btn.repaint ();
			}
		}

	/** getButtons: return the buttons */
	public Vector getButtons ()
		{
		return (d_VButtons);
		}

	/** getButtons: return the buttons */
	public void setTitle (String Title)
		{
		d_Title = new String (s_TITLE_PREFIX + Title);
		}
		

	/** init: initialize the gui building on "d_Frame" frame */
	private void init ()
		{
		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("Initializing GUI (" + d_LPP.size() + 
						") ...");

		// declare local variables 
		Double			d=buttons_per_row ();
		int			numbtns=d.intValue()*d.intValue();
		JAC_ImageTextButton	Btn; 
		Image			image; 
		MenuBar			Menubar = new MenuBar ();
		Menu			FileMenu = new Menu ("File");
		Menu			LParserMenu = new Menu ("Population");
		Menu			AboutMenu = new Menu ("About");

		d_Frame = new Frame (d_Title);
		d_numButtons = d.intValue()*d.intValue();
		d_Frame.setLayout (new GridLayout (d.intValue(), d.intValue()));
		FileMenu.add (d_FileNew);
		d_FileNew.addActionListener (this);
		FileMenu.add (d_FileOpen);
		d_FileOpen.addActionListener (this);
		FileMenu.add (d_FileSave);
		d_FileSave.addActionListener (this);
		FileMenu.addSeparator ();
		FileMenu.add (d_FilePrefs);
		d_FilePrefs.addActionListener (this);
		FileMenu.addSeparator ();
		FileMenu.add (d_FileExit);
		d_FileExit.addActionListener (this);

		LParserMenu.add (d_LParserBreed);
		d_LParserBreed.addActionListener (this);

		AboutMenu.add (d_AboutAbout);
		d_AboutAbout.addActionListener (this);

		Menubar.add (FileMenu);
		Menubar.add (LParserMenu);
		Menubar.add (AboutMenu);

		d_Frame.setMenuBar (Menubar);
		d_Frame.setSize (new Dimension (800, 600));

		// generate the buttons 
		for (int i=0; i<numbtns; i++)
			{
			String	s=null;

			if (i < d_LPP.size())
				{
				LParserObj	LPO = (LParserObj)d_LPP.elementAt(i);
				//image = Toolkit.getDefaultToolkit().getImage (name);
				image=null;
				s = new String (s_INIT_BTN);
				}
			else
				{
				//image = Toolkit.getDefaultToolkit().getImage ("test.jpg");
				image = null;
				}

			Btn = new JAC_ImageTextButton (s, image);

			Btn.addMouseListener (this);
			d_VButtons.addElement (Btn);
			d_Frame.add (Btn);
			}

		//pack ();

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("GUI Setup Finished ...");

		d_Frame.show ();
		}

	/** actionPerformed: implement menu-event method of ActionListener */
	public void actionPerformed (ActionEvent e)
		{
		Object		obj = e.getSource ();
		String		Fname=null, Dname=null; // File & Dir names 

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("ActionEvent: " + e.toString());

		if (obj == d_FileNew)
			{
			//LPEvol LPE = (LPEvol)d_CallingClass.clone ();
			//LPE.updNewClone ();
			}

		else
		if (obj == d_FileOpen)
			{
			JAC_FilterEndsWith 	JFilter = new JAC_FilterEndsWith (".lsp");

			d_FD = new FileDialog(d_Frame, "Open File", FileDialog.LOAD);
			d_FD.setFilenameFilter (JFilter);
			d_FD.show ();
			// should have a filename filter here
			Fname = d_FD.getFile ();
			Dname = d_FD.getDirectory ();

			if (Fname!=null && Dname!=null)
				{
				int	n;

				d_LPP = new LParserPop ();
				if (d_LPP.size() > 0)
					{
					if (LPEOptions.s_recycleImages)
						{
						n = recycleImages ();
						if (n < d_LPP.size())
							d_LPP.render (d_VButtons, false, true, false);
						}
					else
						d_LPP.render (d_VButtons);
					}
				}
			}

		else
		if (obj == d_FileSave)
			{
			d_FD=new FileDialog(d_Frame, "Save File", FileDialog.SAVE);
			d_FD.show ();
			Fname = d_FD.getFile ();
			Dname = d_FD.getDirectory ();

			if (Fname!=null && Dname!=null)
				{
				//
				}
			}

		else
		if (obj == d_FilePrefs)
			{
			// LPEGuiOptions Lpego = new LPEGuiOptions (d_Frame, true);
			System.out.println ("Preferences are not done yet");
			}

		else
		if (obj == d_FileExit)
			{
			d_Frame.dispose ();
			System.exit (0);
			}

		else
		if (obj == d_LParserBreed)
			{
			LParserPop		ParentPop=d_LPP;
			LParserObj		L=null;
			JAC_ImageTextButton	B=null;
			int			c=0,
						limb=d_VButtons.size(),
						limp=d_LPP.size();

			LParserGA LPGA=new LParserGA ((LParserObj)d_LPP.elementAt(0));	// anything will do 
			LPGA.setChromOffset (LPEOptions.s_chromOffset);
			LPGA.setNumberBreed (LPEOptions.s_numberBreed, LPEOptions.s_numberBreedProb);
			LPGA.setCrossBeforeNumber (LPEOptions.s_crossBeforeNumber);
			LPGA.setLimitBrackets (LPEOptions.s_limitBrackets);
			LPGA.setAppend (LPEOptions.s_append, LPEOptions.s_appendReplace);

			// build population 
			LParserPop newLPP = LPGA.breed (d_LPP, d_numButtons);
			newLPP.setName (d_LPP.getName());
			d_LPP = newLPP;

			// update button labels
			while (c<limb && c<limp)
				{
				L = (LParserObj)d_LPP.elementAt(c);
				B = (JAC_ImageTextButton)d_VButtons.elementAt(c);
				B.setLabel (L.fullName());
				c++;
				}
			d_Frame.setTitle (new String (s_TITLE_PREFIX + 
				d_LPP.generationName()));
			
			d_LPP.write ();
			d_LPP.render (d_VButtons, false, true, true);
			}

		else
		if (obj == d_AboutAbout)
			{
			}
		}

	/** actionPerformed: implement mouse-event method of MouseListener */
	public void mouseClicked (MouseEvent e)
		{
		int			lim = d_VButtons.size ();
		Object			obj = e.getSource ();
		JAC_ImageTextButton 	Btn;
		LParserObj			LPO;
		int			mod = e.getModifiers ();
		
		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("MouseEvent: " + e.toString());

		for (int i=0; i<lim; i++)
			if (obj.equals (d_VButtons.elementAt (i)))
				{
				if (JAC_GlobalDebug.debug() > 0)
					System.out.println ("Button " + i + " clicked ...");

				if (d_LPP.size() > i)
				    {
				    LPO = (LParserObj)d_LPP.elementAt(i);

				    if (mod == e.BUTTON1_MASK)
					{
					LPO.setSelected (!LPO.getSelected());
					Btn=(JAC_ImageTextButton)d_VButtons.elementAt(i);
					if (!Btn.getLabel().equals (LPO.fullName()))
						Btn.setLabel (LPO.fullName());
					if (LPO.getSelected())
						Btn.setShowLabel (true);
					else
						Btn.setShowLabel (false);
					}
				    else
					{
					int		clim = LPO.size();
					String		Gs = new String (""),
							Cs = null;
					JAC_SimpleBufferEdit	STE=null; 

					for (int j=0; j<clim; j++)
						{
						Cs = (String)LPO.elementAt(j);
						Gs = new String (Gs + Cs + "\n");
						}
					Gs = new String (Gs + "@");
					System.out.println (Gs);
					try {
						STE = new JAC_SimpleBufferEdit (Gs.toCharArray());
						//System.out.println ("HERE");
						STE.start ();
						STE.join (); 
						}
					catch (InterruptedException ie) {
						System.out.println ("Caught exception in LPEGui::mouseClicked: " + ie); 
						}

					//Cs = STE.close ();
					//System.out.println ("HERE2");
					}
				    }
				
				i=lim;
				}
		}

	/** Remaining, non-implemented methods in MouseListener */
	public void mouseEntered (MouseEvent e)		{ }
	public void mouseExited (MouseEvent e)		{ }
	public void mousePressed (MouseEvent e)		{ } 
	public void mouseReleased (MouseEvent e)	{ } 

	/** recycle_images: try to load the images of a population 
		by scavenging for leftovers of previously renderd pops. 
	**/
	public int recycleImages ()
		{
		Vector		VFNames;
		JAC_FileReader	Jfr;
		LParserObj	LPO;
		String		Basename;
		JAC_FileUtil	Fls, Fjpg;
		int		c = 0;

		// process the list 
		int lim = d_LPP.size ();
		int blim = d_VButtons.size ();
		for (int i=0; i<lim && i<blim; i++)
		    {
		    try {
			JAC_ImageTextButton	Btn;

			Basename=JAC_FileUtil.baseName (((LParserObj)d_LPP.elementAt(i)).getName());

			if (JAC_GlobalDebug.debug() > 0)
				System.out.print ("Checking recycleImages: " + Basename);

			Fls = new JAC_FileUtil (new String (Basename + ".ls"));
			Fjpg = new JAC_FileUtil (new String (Basename + ".jpg"));
			if (Fls.exists() && Fjpg.exists() && 
			    Fjpg.isNewer (Fls.d_Fname))
				{
				Btn = (JAC_ImageTextButton)d_VButtons.elementAt(i);
				Btn.setImage (Fjpg.d_Fname);
				Btn.setShowLabel (false);
				c++;
				if (JAC_GlobalDebug.debug() > 0)
					System.out.println (" ok");
				}
			else
				{
				if (JAC_GlobalDebug.debug() > 0 &&
				    (Fjpg.exists() && !Fjpg.isNewer(Fls.d_Fname)))
					System.out.println (" oops, wrong timestamp");
				else
				if (JAC_GlobalDebug.debug() > 0)
					System.out.println (" no file");
				}
			}
		    catch (Exception e) {
			if (JAC_GlobalDebug.verbose())
				System.out.println ("Caught exception in LPEGui::recycleImages():" + e);
		   	 }
		    }
		return (c);
		}

	/** buttons_per_row: returns the rows and columns of the number of 
		buttons which will best fit the population size 
	**/
	private Double buttons_per_row ()
		{
		Double			D;

		if (d_LPP == null)
			D = new Double (4.0);		// 9 buttons 
		else
			{
			if (d_LPP.size() < 16)		// 4 buttons 
				D = new Double (4.0);
			else				// next best fit 
				{
				Integer	I=new Integer (d_LPP.size());
				Double	TD=new Double (java.lang.Math.sqrt (I.doubleValue()));

				if (TD.doubleValue() != TD.intValue())
					D = new Double ((TD.intValue()+1));
				else
					D = TD;
				}
			}

		if (JAC_GlobalDebug.debug() > 0)	
			System.out.println ("Determining # of rows x columns: " 
				+ D + "x" + D);

		return (D);
		}
}

