/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.io.*;
import java.awt.*;
import java.util.*;


/* ************************************************************************** */
/* *** LPEOptions: Class to hold options for LPEVol. Variables are global *** */
/* *************** and used by all the LPE*.java modules ******************** */
/* ************************************************************************** */
public class LPEOptions 
{
	// Global constants
	public static final long 	s_KB=1024;
	public static final long 	s_MB=s_KB*s_KB;
	
	// Option variables
	public static float		s_append=(float)0;
	public static float		s_appendReplace=(float)0.5;
        public static boolean		s_autostart=true;
        public static boolean		s_batch=false;
        public static int		s_renderMode=LPERender.s_RENDER_MODE_INC;
	public static int		s_chromOffset=LParserGA.s_CHROM_OFFSET;
        public static boolean		s_crossBeforeNumber=true;
	public static String		s_libdir=null;
	public static int		s_limitBrackets=LParserGA.s_LIMIT_BRACKETS_ALL;
        public static long		s_minincfile=50*s_KB;
        public static long		s_maxincfile=5*s_MB;
        public static long		s_maxgen=Long.MAX_VALUE;
        public static int		s_mutate=0;
        public static boolean		s_cleanup=true;
        public static boolean		s_directory=true;
        public static boolean		s_gui=true;
        public static boolean		s_recycleImages=true;
        public static boolean		s_write=true;
	public static int		s_numberBreed=0;
	public static float		s_numberBreedProb=(float)0.5;
	public static boolean		s_numberBreedInt=false;
        public static int		s_popsize=16;
	public static boolean		s_renderBig=false;
	public static boolean		s_replaceBroken=false;
	public static boolean		s_replaceSize=false;
	public static int		s_maxreplace=10;
        public static boolean		s_quit=false;

	// enables us to read and store a population 
	private LParserPop		d_LPP=null;


	/** Constructor: init population and set args=null */
	public LPEOptions (LParserPop LPP)
		{
		this (LPP, null);
		}

	/** Constructor: init population, process command line arguments */
	public LPEOptions (LParserPop LPP, String[] args)
		{
		if (LPP == null)
			JAC_Bailout.bailout ("Sanity Check failed in LPEOptions(): LPP==null ... Exiting");

		d_LPP = LPP;
		if (args != null)
			parseArgs (args);
		}

	/** ParseArgs: process the command line arguments */
	public void parseArgs (String[] args)
		{
		int	rc;

		for (int i=0; i<args.length; i++)
		   {
		   // help
		   if (args[i].equals ("-?") || args[i].equals ("-help"))
			usage ();

		   else
		   // append probability
		   if (args[i].equals ("-append"))
			{
			if (args.length > i+1)
				{
				s_append = new Float(args[++i]).floatValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -append option: append=" + s_append);
				}
			else
				{
				System.out.println ("ParseArgs() error: -append not followed by <float>");
				System.exit (0);
				}
			}

		   else
		   // append replace probability
		   if (args[i].equals ("-append_replace"))
			{
			if (args.length > i+1)
				{
				s_appendReplace = new Float(args[++i]).floatValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -append_replace option: appendReplace=" + s_appendReplace);
				}
			else
				{
				System.out.println ("ParseArgs() error: -append_replace not followed by <float>");
				System.exit (0);
				}
			}

		   else
		   // run LPEvol in batch mode
		   if (args[i].equals ("-batch"))
			{
			s_batch = true;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -batch option: batch=" + s_batch);
			}

		   else
		   // produce blob povray output
		   if (args[i].equals ("-blob"))
			{
			s_renderMode = LPERender.s_RENDER_MODE_BLOB;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -blob option: renderMode=" + s_renderMode);
			}

		   else
		   // assign the chromosome offset to observe when breeding 
		   if (args[i].equals ("-chrom_offset"))
			{
			if (args.length > i+1)
				{
				s_chromOffset = new Integer(args[++i]).intValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -chrom_offset option: s_chromOffset=" + s_chromOffset);
				}
			else
				{
				System.out.println ("ParseArgs() error: -chrom_offset not followed by number");
				System.exit (0);
				}
			}

		   else
		   // cross after first guessing a number
		   if (args[i].equals ("-cross_after") || 
		       args[i].equals ("-cross_after_number") )
			{
			s_crossBeforeNumber = false;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -cross_after option: s_crossBeforeNumber=" + s_crossBeforeNumber);
			}

		   else
		   // set libdir (the directory where we can find our _* files)
		   if (args[i].equals ("-libdir") || 
		       args[i].equals ("-L") )
			{
			if (args.length > i+1)
				{
				s_libdir = args[++i];
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -libdir option: s_libdir=" + s_libdir);
				}
			else
				{
				System.out.println ("ParseArgs() error: -libdir not followed by directory");
				System.exit (0);
				}
			}

		   else
		   // turn of 'limit brackets'
		   if (args[i].equals ("-limit_brackets"))
			{
			if (args.length > i+1)
				{
				s_limitBrackets = new Integer(args[++i]).intValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -limit_brackets option: s_limitBrackets=" + s_limitBrackets);
				}
			else
				{
				System.out.println ("ParseArgs() error: -limit_brackets not followed by number");
				System.exit (0);
				}
			}

		   else
		   // maximum size of generated povray include file
		   if (args[i].equals ("-maxincfile"))
			{
			if (args.length > i+1)
				{
				s_maxincfile = new Long(args[++i]).longValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -maxincfile option: maxincfile=" + s_maxincfile);
				}
			else
				{
				System.out.println ("ParseArgs() error: -maxincfile not followed by number");
				System.exit (0);
				}
			}

		   else
		   if (args[i].equals ("-maxgen"))
			{
			if (args.length > i+1)
				{
				s_maxgen=new Long(args[++i]).longValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -maxgen option: max_generations=" + s_maxgen);
				}
			else
				{
				System.out.println ("ParseArgs() error: -maxgen not followed by number");
				System.exit (0);
				}
			}

		   else
		   if (args[i].equals ("-maxreplace"))
			{
			if (args.length > i+1)
				{
				s_maxreplace=new Integer(args[++i]).intValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -maxreplace option: maxreplace=" + s_maxreplace);
				}
			else
				{
				System.out.println ("ParseArgs() error: -maxreplace not followed by number");
				System.exit (0);
				}
			}

		   else
		   // produce blob povray output
		   if (args[i].equals ("-mblob"))
			{
			s_renderMode = LPERender.s_RENDER_MODE_MBLOB; 
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -mblob option: renderMode=" + s_renderMode);
			}

		   else
		   if (args[i].equals ("-mutate"))
			{
			if (args.length > i)
				{
				s_mutate=new Integer(args[++i]).intValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -mutate option: max_generations=" + s_mutate);
				}
			else
				{
				System.out.println ("ParseArgs() error: -mutate not followed by number");
				System.exit (0);
				}
			}

		   else
		   // automatically start rendering upon startup
		   if (args[i].equals ("-noautostart"))
			{
			s_autostart = false;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -noautostart option: autostart=" + s_autostart);
			}

		   else
		   if (args[i].equals ("-noclean"))
			{
			s_cleanup = false;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -noclean option: cleanup=" + s_cleanup);
			}

		   else
		   if (args[i].equals ("-nodir"))
			{
			s_directory = false;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -nodir option: s_directory=" + s_directory);
			}

		   else
		   if (args[i].equals ("-nogui"))
			{
			s_gui = false;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -nogui option: gui=" + s_gui);
			}

		   else
		   if (args[i].equals ("-norecycle"))
			{
			s_recycleImages = false;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -norecycle option: s_recycleImages=" + s_recycleImages);
			}

		   else
		   if (args[i].equals ("-nowrite"))
			{
			s_write = false;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -nowrite option: write=" + s_write);
			}

		   else
		   // assign the number offset to observe when breeding 
		   if (args[i].equals ("-numeric"))
			{
			if (args.length > i+1)
				{
				s_numberBreed = new Integer(args[++i]).intValue();
				s_numberBreedProb = new Float(args[++i]).floatValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -numeric option: s_numberBreed=" + s_numberBreed);
				}
			else
				{
				System.out.println ("ParseArgs() error: -numeric not followed by <number> <float>");
				System.exit (0);
				}
			}

		   else
		   if (args[i].equals ("-numericInt"))
			{
			s_numberBreedInt = true;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -numericInt option: s_numberBreedInt=" + s_numberBreedInt);
			}


		   else
		   // specify a file containing a file list 
		   if (args[i].equals ("-pfile"))
			{
			if (args.length > i)
				{
				if ((rc=d_LPP.popFile(args[++i])) <= 0)
					{
					System.out.println ("Error reading population file: rc=" + rc + " ... Exiting");
					System.exit (rc);
					}
				}
			else
				{
				System.out.println ("ParseArgs() error: -pfile not followed by filename");
				System.exit (0);
				}
			}

		   else
		   if (args[i].equals ("-psize"))
			{
			if (args.length > i)
				{
				s_popsize = new Integer(args[++i]).intValue();
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -popsize option: max_pop_size=" + new Integer(args[i]));
				}
			else
				{
				System.out.println ("ParseArgs() error: -psize not followed by population size ");
				System.exit (0);
				}
			}

		   else
		   if (args[i].equals ("-quit"))
			{
			s_quit = true;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -quit option: quit=" + s_quit);
			}

		   else
		   if (args[i].equals ("-render_big"))
			{
			s_renderBig = true;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -render_big option: render_big=" + s_renderBig);
			}


		   else
		   if (args[i].equals ("-replace"))
			{
			s_replaceBroken = s_replaceSize = true;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -replace option: s_replaceSize=s_replaceBroken=" + s_replaceSize);
			}

		   else
		   if (args[i].equals ("-replace_broken"))
			{
			s_replaceBroken = true;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -noreplace option: s_replaceBroken=" + s_replaceBroken);
			}

		   else
		   if (args[i].equals ("-replace_size"))
			{
			s_replaceBroken = true;
			if (JAC_GlobalVerbose.verbose())
				System.out.println ("Caught -replace_size option: s_replaceSize=" + s_replaceSize);
			}


/** 
		   - seed moved to LPEvol.java for corect batch seeding

		   else
		   if (args[i].equals ("-seed"))
			{
			if (args.length > i)
				{
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -seed option: seed=" + new Long(args[i+1]));
				JAC_GlobalRandom.init (new Long(args[++i]).longValue());
				}
			else
				{
				System.out.println ("ParseArgs() error: -seed not followed by seed");
				System.exit (0);
				}
			}
**/ 

		   else
		   // verbose
		   if (args[i].equals ("-v") || args[i].equals ("-verbose"))
			{
			// we already handle this in the constructor 
			}

		   else
		   // debug 
		   if (args[i].startsWith ("-dbg"))
			{
			// we already handle this in the constructor 
			}

		   else
		   if (args[i].equals ("-bench"))
			{
			// we already handle this in the constructor 
			}

		   else
		   if (args[i].startsWith ("-"))
			{
			System.out.println ("ParseArgs() error: " + args[i] + " not a recognized option ...");
			usage(); 
			}

		   // default: arg[i] is an LParser population file
		   else	
		   if ((rc=d_LPP.popFile(args[i])) != 0)
			{
			System.out.println ("Error reading population file: rc=" + rc + " ... Exiting");
			System.exit (rc);
			}
		   }

		argsSanityCheck ();
		}

	/** usage: make sure we have reasonable arguments */
	private void argsSanityCheck ()
		{
		// check if -libdir is a directory 
		if (s_libdir != null)
			{
			File	F = new File (s_libdir);

			if (!F.isDirectory())
				JAC_Bailout.bailout ("argsSanityCheck() error: -libdir (" + s_libdir + ") is not a directory ... ");
			}

		// -nowrite issued without -batch ... 
		if (!s_write && !s_batch)
			JAC_Bailout.bailout ("argsSanityCheck() error: -nowrite must be issued with -batch ... ");

		// -nowrite issued without -batch ... 
		if (!s_autostart && s_batch)
			JAC_Bailout.bailout ("argsSanityCheck() error: -noautostart doesn't make sense with -batch ...");

		// -quit issued with -autostart ...
		if (s_quit && !s_autostart)
			JAC_Bailout.bailout ("argsSanityCheck() error: -quit doesn't make sense with -noautostart ...");

		// -quit issued with -nogui ...
		if (s_quit && !s_gui)
			JAC_Bailout.bailout ("argsSanityCheck() error: -quit doesn't make sense with -nogui ...");

		// invalid -limit_brackets option ...
		if (s_limitBrackets < LParserGA.s_LIMIT_BRACKETS_OFF || 
		    s_limitBrackets > LParserGA.s_LIMIT_BRACKETS_RIGHT)
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -limit_brackets argument (!(0<lb<" + LParserGA.s_LIMIT_BRACKETS_RIGHT + ")) ...");

		// invalid -append (probability) value ...
		if (s_append < 0 || s_append > 1)
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -append argument (!(0<append<1)) ...");

		// invalid -append (replace probability) value ...
		if (s_appendReplace <= 0 || s_appendReplace >= 1)
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -append replace argument (!(0<=AR<=1)) ...");

		// invalid -breed_number (replace probability) value ...
		if (s_numberBreed < 0 || s_numberBreed > LParserObj.s_NUM_VALUES)
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -numeric <number> argument (!(0<AR<=1)) ...");

		// invalid -breed_number (replace probability) value ...
		if (s_numberBreedProb <= 0 || s_numberBreedProb > 1)
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -numeric <probability> argument (!(0<AR<=1)) ...");

		// check for invalid negative value for -chrom_offset
		if (s_chromOffset < 0)
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -chrom_offset argument (!(n>=0))...");

		// check for invalid negative value for -chrom_offset
		if (s_chromOffset < 0)
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -chrom_offset argument (!(n>=0))...");

		// check for invalid negative value for -maxincfile 
		if (s_maxincfile < 10000) 
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -maxincfile argument (!(n>=1000))...");

		// check for invalid negative value for -mutate 
		if (s_mutate < 0)
			JAC_Bailout.bailout ("argsSanityCheck() error: illegal -mutate argument (!(n>=0))...");
		}

	/** usage: return usage information */
	public static void usage ()
		{
		String	Empty=new String ();

		System.out.println (Empty);
		System.out.println ("Usage: LPEvol [arg1] ... [argn]");
		System.out.println ("Valid command line options are:");
		System.out.println ("  [-?|-help]		help");
		System.out.println ("  -append <arg> 	append new chromosomes with  p 0<=p<=1");
		System.out.println ("  -append_replace <arg>	update old references after append with  p 0<p<1");
		System.out.println ("  -batch		run program in batch (non-interactive) mode");
		System.out.println ("  -blob			generate single PovRay blob file");
		System.out.println ("  -chrom_offset <arg> 	number of chromosomes to skip when breeding");
		System.out.println ("  -cross_after		shift right when a position guess hits a number"); 
		System.out.println ("  [-libdir|-L] <arg>	directory where _*.pov files are located"); 
		System.out.println ("  -limit_brackets <arg>	limit bracket mode (value modes are 0-3)");
		System.out.println ("  -maxincfile <arg> 	maximum size of PovRay include file to be rendered");
		System.out.println ("  -maxgen <arg>		maximum number of generations (batch mode)");
		System.out.println ("  -maxreplace <arg>	maximum number times to replace a broken org");
		System.out.println ("  -mblob		generate multiple PovRay blob files");
		System.out.println ("  -mutate <arg>		number of times lparser should mutate");
		System.out.println ("  -noautostart		do not autostart rendering population upon startup");
		System.out.println ("  -noclean		don't remove inlude files");
		System.out.println ("  -nodir		don't create population subdirectories");
		System.out.println ("  -nogui		run without gui (batch mode)");
		System.out.println ("  -norecycle	don't recycle images (forces render)");
		System.out.println ("  -nowrite		don't write orgs/pops to file (batch mode)");
		System.out.println ("  -numeric <arg> <arg> 	number of params and breed probability");
		System.out.println ("  -pfile <arg>		specify population file to read");
		System.out.println ("  -psize <arg>		specify population size");
		System.out.println ("  -quit			render population and quit");
		System.out.println ("  -render_big		render a 800x600 image");
		System.out.println ("  -replace		same as -noreplace_size and -noreplace_break");
		System.out.println ("  -replace_size		replace LSystems which are too big (-maxincfile)");
		System.out.println ("  -replace_broken	replace LSystems which break lparser or PovRay");
		System.out.println ("  -seed <arg>		specify random number seed");
		System.out.println ("  [-v|-verbose]		enable verbose mode");
		System.out.println (Empty);
		System.out.println ("LPEvol " + LPEvol.s_version + " - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)");
		System.out.println ("             http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm");
		System.exit (0);
		}
}

