/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.awt.*;
import java.util.*;


/* *********************************************************************** */
/* **** LPERender: Threadable class to take 1 LPO from *.ls to *.jpg **** */
/* *********************************************************************** */
public class LPERender extends Thread
{
	public static final int		s_RENDER_MODE_INC = 0;
	public static final int		s_RENDER_MODE_BLOB = 1;
	public static final int		s_RENDER_MODE_MBLOB = 2;
	public static final int		s_MBLOB_INCLUDE_FILES = 7;

	// LParser commands for normal and blob mode 
	public static final String	s_LPARSER_INC=new String 
						("lparser -vc ");
	public static final String 	s_LPARSER_BLOB=new String 
						("lparser -bc ");
	public static final String 	s_LPARSER_MBLOB=new String 
						("lparser -Bc ");

	// PovRay default file for normal, blob mode and multi blob mode
	public static final String	s_POVFILE_INC=new String 
						("_lsyswrap_i.pov");
	public static final String	s_POVFILE_BLOB=new String 
						("_lsyswrap_b.pov");
	public static final String	s_POVFILE_MBLOB=new String 
						("_lsyswrap_mb.pov");

	private LParserPop		d_LPP=null;
	private LParserPop		d_parentLPP=null;
	private LParserObj		d_LPO=null;
	private LParserObj		d_leftoverReplaceLPO=null;
	private JAC_ImageTextButton	d_Btn=null;
	private Vector			d_VBtn=null;
	private String			d_Basename=null;
	private	String			d_PovFile=s_POVFILE_INC;
	private boolean 		d_renderAll=true;
	private int			d_nreplace=0;	// number of times we've replaced a broken LSystem

	// local copy of global settings 
	private	int			d_mutate=0;
	private boolean			d_cleanup=true;
	private boolean			d_directory=true;
	private int			d_mode=0;
	private	long			d_maxIncFileSize=5*LPEOptions.s_MB;
	private	long			d_minIncFileSize=0;

	// all the different filenames 
	private	String			d_Lsname 	= null,
					d_Incname 	= null,
					d_Povname 	= null,
					d_Tganame 	= null,
					d_Jpgname 	= null,
					d_Pwd 		= null,
					d_Dirname 	= null;

	/** Constructor: do sanity check then assign d_LPO */
	public LPERender (LParserObj LPO, JAC_ImageTextButton Btn)
		{
		super ();
		if (LPO == null)
			JAC_Bailout.bailout ("Sanity Check Failed: LPO==null in LPERender::LPERender() ... Exiting\n", -1);

		d_LPO=LPO;
		d_Btn=Btn;
		init ();
		}

	/** Constructor: do sanity check then assign d_LPO */
	public LPERender (LParserPop LPP, Vector VBtn)
		{
		super ();
		if (LPP == null)
			JAC_Bailout.bailout ("Sanity Check Failed: LPP==null in LPERender::LPERender() ... Exiting\n", -1);

		d_LPP=LPP;
		d_VBtn=VBtn;
		init ();
		}

	/** init: initialize member variables for this instance */
	private void init ()
		{
		LParserObj	LPO = (LParserObj)d_LPP.elementAt(0);

		d_Pwd = System.getProperties().getProperty ("user.dir");
		d_Basename = LPO.fullName();
		d_Dirname = d_LPP.generationName();
		d_mutate = LPEOptions.s_mutate;
		d_cleanup = LPEOptions.s_cleanup;
		d_directory = LPEOptions.s_directory;
		setRenderMode (LPEOptions.s_renderMode);
		this.setName ("LPERender Thread");

		if (d_mode == s_RENDER_MODE_BLOB)
			d_PovFile=s_POVFILE_BLOB;
		else
		if (d_mode == s_RENDER_MODE_MBLOB)
			d_PovFile=s_POVFILE_MBLOB;
		}

	/** run: thread initialization */
	public void run ()
		{
		LParserObj	LPO;
		int		lim;

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("Starting Thread: " + this.getName());

		// since this is potentially long-running, we make a local copy 
		// of LPEOptions.d_[max|min] incfile; make us immune from change
		d_minIncFileSize=LPEOptions.s_minincfile;
		d_maxIncFileSize=LPEOptions.s_maxincfile;

		if (d_LPO != null)
			povRender (d_LPO);
		else
			{
			try {
			    if (d_directory)
				{
				JAC_FileUtil Fj = new JAC_FileUtil (d_Pwd, new String (d_Dirname+".lsp"));
				Fj.moveTo (d_Dirname);
				} 
			    }
			catch (Exception e) 
			    { System.out.println ("Caught exception in LPERender::run(): " + e); }

			lim = d_LPP.size();
			for (int i=0; i<lim; i++)
				{
				LPO=(LParserObj)d_LPP.elementAt(i);
				if (d_VBtn != null)
					d_Btn=(JAC_ImageTextButton)d_VBtn.elementAt(i);

				if (d_renderAll || 
				    (!d_renderAll && d_Btn.getImage()==null))
					{
					d_nreplace=0;
					d_leftoverReplaceLPO=null;
					povRender (LPO);
					}
				}
			}

		// if this is generation 0, make a copy of the JPGs 
		// under the original name read from the *.lsp file
		if (((LParserObj)d_LPP.elementAt(0)).getGeneration() == 0)
			copyPop1Jpg ();

		if (JAC_GlobalVerbose.verbose())
			System.out.println ("Finished rendering population ...");
		}

	/** setupRenderNames: construct the file names based on Basename */
	public void setupRenderNames (String Basename)
		{
		d_Basename=Basename;
		d_Lsname = new String (d_Basename + ".ls");
		d_Incname = new String (d_Basename + ".inc");
		d_Povname = new String (d_Basename + ".pov");
		d_Tganame = new String (d_Basename + ".tga");
		d_Jpgname = new String (d_Basename + ".jpg");
		}

	/** setRenderAll: do we have to rerender existing images? */
	public void setRenderAll (boolean renderAll)
		{
		d_renderAll = renderAll;
		}

	/** setLParserMode: set the lparser mode to a valid value */
	public void setRenderMode (int mode)
		{
		if (mode >= s_RENDER_MODE_INC && 
		    mode <= s_RENDER_MODE_MBLOB)
			d_mode = mode;
		else
			System.err.println ("Warning: Mode out of range (" + mode + ") in LPERender::SetLParserMode()");
		}

	/** setMaxFileSize: set the maximum allowed povray include file size */
	public void setMaxFileSize (long	maxsize)
		{
		if (maxsize < d_minIncFileSize)
			{
			System.out.println ("Sanity check Failed: maxsize < " + 
				d_minIncFileSize + " (min) ... Ignoring");
			}
		else
			d_maxIncFileSize = maxsize;
		}

	/** povRender: go through the steps of processing the files for LPO */
	private int povRender (LParserObj _LPO_)
		{
		int		rc=0;
		long		flen=0;
		Image		I=null;
		File		Lpf;
		LPEFileUtil	LFU=null;
		LParserObj	LPO=_LPO_;

		if (LPO == null)
			return (-1);

		setupRenderNames (LPO.fullName());

		try {
		    rc=lparserCommand ();

		    // we have output[1-7].inc to process
		    if (d_mode == s_RENDER_MODE_MBLOB)
			{
			long		max=0;

			for (int i=1; i<=s_MBLOB_INCLUDE_FILES; i++)
				{
				Lpf = new File (new String ("output" + i + ".inc"));
				if (Lpf.length() > max)
					max = Lpf.length();
				}

			flen = max;
			}
		    else
			// process d_Incname
			{
			Lpf = new File (d_Incname);

			if (!Lpf.exists())
				{
				if (rc == 0)
					return (-1);
				}
			else
				flen = Lpf.length();
			}

		    // check for file size limit 
		    if (flen < d_maxIncFileSize)
			{
			if (rc == 0)
			    {
			    rc=createPovrayFile ();
			    rc=renderPovrayFile ();
			    if (rc == 0)
				rc=convertTGAFile ();

			    LFU = new LPEFileUtil (d_Dirname, d_Basename);
			    LFU.cleanupFiles ();

			    // if everything is OK, get&assign the new button image	
			    if (rc == 0)
			    	{
			    	if (d_Btn!=null)
				    {
				    if (d_directory)
					d_Btn.setImage (new String (d_Dirname + File.separator + d_Jpgname));
				    else
					d_Btn.setImage (d_Jpgname);
				    d_Btn.setLabel (LPO.fullName());
				    d_Btn.setShowLabel (false);
				    d_Btn.setVisible (true);
				    }
			        }
			    }

			if (rc != 0)
			    {
			    if (d_Btn!=null)
				{
				d_Btn.setImage ((Image)null);
				// if (rc == 139) 
				//	d_Btn.setLabel (new String ("Error = " + rc + " (Core Dump)")); 
				// else 
				d_Btn.setLabel (new String ("Error = " + rc)); d_Btn.setVisible (true);

				// replace the broken LSystem 
				if (LPEOptions.s_replaceBroken &&
				    d_nreplace < LPEOptions.s_maxreplace )
					{
					replaceBrokenOrg (LPO);
					d_nreplace++;
					}
				}
			     }
			}
		    else
			{
			if (JAC_GlobalDebug.verbose())
				System.out.println ("Skipping " + d_Incname + " (size=" + flen + ", max=" + d_maxIncFileSize + ")");

			if (LPEOptions.s_replaceSize)
				{
				d_Btn.setLabel ("Replace: " + flen + " > " + d_maxIncFileSize);
				d_Btn.setVisible (true);
				if (d_nreplace < LPEOptions.s_maxreplace)
					{
					replaceBrokenOrg (LPO);
					d_nreplace++;
					}
				}
			else
				{
				d_Btn.setImage ((Image)null);
				d_Btn.setLabel ("Too large: " + flen + " > " + d_maxIncFileSize);
				d_Btn.setVisible (true);
				}
			}

		    // set org and button status
		    LPO.setSelected (false);
		    LFU = new LPEFileUtil (d_Dirname, d_Basename);
		    LFU.cleanupFiles ();
		    }
		catch (InterruptedException e) 
		    {
		    if (d_Btn!=null)
			{
			d_Btn.setImage ((Image)null);
			d_Btn.setLabel ("Error: File not found");
			d_Btn.setVisible (true);
			if (JAC_GlobalDebug.verbose())
				System.err.println("Caught Exception in LPERender::povRender():" + e);
			}
		    else
			System.err.println("Caught Exception in LPERender::povRender():" + e);
		    }
		catch (IOException e) {
			System.err.println("Caught Exception in LPERender::povRender():" + e);
		     	}

		return (0);
		}

	/** replaceBrokenOrg: rebreed an organism to overwrite 
	    the LPO which breaks LParser or PovRay
	*/
	private void replaceBrokenOrg (LParserObj LPO) throws FileNotFoundException
		{
		File		F;
		LParserObj	Parent1, Parent2, ReplaceOrg;
		LParserObj 	[]P; 
		LParserGA	LPGA;
		LParserPop	PPop = new LParserPop(),
				CPop = null;

		// cleanup old files
		F = new File ("core");
		if (F.exists())
			F.delete ();
		F = new File ("output.inc");
		if (F.exists())
			F.delete ();
		F = new File (LPO.fullName());
		if (F.exists())
			F.delete ();

		// we have an org left over from the last replace breed
		if (d_leftoverReplaceLPO == null)
			{
			if (JAC_GlobalDebug.verbose())
				System.out.println ("Breeding org for replacement");

			P = LPO.getParents ();

			Parent1 = P[0];
			Parent1.setSelected (true);
			PPop.add (Parent1);

			Parent2 = P[1];
			Parent2.setSelected (true);
			PPop.add (Parent2);

			// overwrite LPO with replacement and write new *.ls file 
			LPGA = new LParserGA (LPO);	// dummy constructor 
			LPGA.setChromOffset (LPEOptions.s_chromOffset);
			LPGA.setNumberBreed (LPEOptions.s_numberBreed, LPEOptions.s_numberBreedProb);
			LPGA.setCrossBeforeNumber (LPEOptions.s_crossBeforeNumber);
			LPGA.setLimitBrackets (LPEOptions.s_limitBrackets);
			LPGA.setAppend (LPEOptions.s_append, LPEOptions.s_appendReplace);
			CPop = LPGA.breed (PPop, 1, false);
			ReplaceOrg = (LParserObj)CPop.elementAt(0);
			d_leftoverReplaceLPO = (LParserObj)CPop.elementAt(1);
			}
		else
			{
			if (JAC_GlobalDebug.verbose())
				System.out.println ("Using cached org for replacement");
			ReplaceOrg = d_leftoverReplaceLPO;
			}

		LPO.copy (ReplaceOrg);
		LPO.write ();

		if (JAC_GlobalDebug.verbose())
			System.out.println ("Replacing broken " + LPO.fullName() + 
				" with newly bred " + ReplaceOrg.fullName());

		// render
		povRender (LPO);
		}

	/** lparserCommand: issue the command to process the *.ls file */
	private int lparserCommand () throws InterruptedException, IOException
		{
		String 		ts=null, lparsercmd = null;
		File		F;
		Process		P;

		// set blob mode 
		if (d_mode == s_RENDER_MODE_INC)
			ts = s_LPARSER_INC;
		else
		if (d_mode == s_RENDER_MODE_BLOB)
			ts = s_LPARSER_BLOB;
		else
		if (d_mode == s_RENDER_MODE_MBLOB)
			ts = s_LPARSER_MBLOB;
		else
			JAC_Bailout.bailout ("Sanity Check Failed in LPERender::createPovrayFile: d_mode out of range ...", -1);

		// append mutate option
		if (d_mutate > 0)
			ts = new String (ts + " -u " + d_mutate + " ");

		// final command: append *.ls filename 
		lparsercmd = ts.concat (d_Basename);

		if (d_Btn!=null)
			{
			d_Btn.setLabel ("LParsing ...");
			d_Btn.setVisible (true);
			}

		if (JAC_GlobalDebug.debug() == 3)
			System.out.println ("\n" + "Executing: " + lparsercmd); 

		P = Runtime.getRuntime().exec (lparsercmd);
		this.yield ();
		P.waitFor ();

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println("Lparser return code: "+ P.exitValue());

		// output file is "output.inc": rename to d_Incname
		if (d_mode != s_RENDER_MODE_MBLOB)
			{
			F = new File ("output.inc");
			if (!F.exists())
				{
				System.err.println("Warning: Can't open output.inc in LPERender::lparserCommand() ... ");
				return (-1);
				}
			F.renameTo (new File (d_Incname));
			}

		if (d_mutate > 0)
			{
			F = new File ("mutation.ls");
			F.delete ();
			}
		return (P.exitValue());
		}

	/** createPovrayFile: create a POV file to trace this LPO */
	private int createPovrayFile () throws InterruptedException, 
					FileNotFoundException, IOException
		{
		Thread			T;
		PovMaxVector	 	Pmv=null;
		double[] 		dmin=null,
					dmax=null;

		if (d_Btn!=null)
			{
			d_Btn.setLabel ("Sizing Povray include file ...");
			d_Btn.setVisible (true);
			}

		// determine the max vectors of the file 
		if (d_mode == LPERender.s_RENDER_MODE_INC)
			{
			Pmv = new PovMaxVector_inc (d_Incname);
			T = new Thread (Pmv, "PovRay Render - Include");
			Pmv.setThread (T);
			T.start ();
			T.join ();
			}
		else
		if (d_mode == LPERender.s_RENDER_MODE_BLOB)
			{
			Pmv = new PovMaxVector_blob (d_Incname);
			T = new Thread (Pmv, "PovRay Render - Blob");
			Pmv.setThread (T);
			T.start ();
			T.join ();
			}
		else
		if (d_mode == LPERender.s_RENDER_MODE_MBLOB)
			{
			PovMaxVector		Tmv;

			Pmv = new PovMaxVector_blob ("output0.inc");
			T = new Thread (Pmv, "PovRay Render - Multi Blob");
			Pmv.setThread (T);
			T.start ();
			T.join ();

			for (int i=1; i<=s_MBLOB_INCLUDE_FILES; i++)
				{
				Tmv = new PovMaxVector_blob (new String ("output" + i + ".inc"));
				T = new Thread (Tmv, "PovRay Render - Multi Blob");
				Tmv.setThread (T);
				T.start ();
				T.join ();
				Pmv.merge (Tmv);
				}
			}
		else
			JAC_Bailout.bailout ("Sanity Check Failed in LPERender::createPovrayFile: d_mode out of range ...", -1);

		dmin = Pmv.getMinValues();
		dmax = Pmv.getMaxValues();

		if (d_Btn!=null)
			{
			d_Btn.setLabel ("Creating Povray File ...");
			d_Btn.setVisible (true);
			}

		if (JAC_GlobalDebug.debug() == 3)
			Pmv.print();

		float	xmin = (float)dmin[0],
			xmax = (float)dmax[0],
			ymin = (float)dmin[1],
			ymax = (float)dmax[1],
			zmin = (float)dmin[2],
			zmax = (float)dmax[2], 
			xrange = xmax-xmin,
			yrange = ymax-ymin,
			zrange = zmax-zmin,
			xcenter = xrange/2+xmin,
			ycenter = yrange/2+ymin,
			zcenter = zrange/2+zmin;

		Vector VOrg=new Vector (5, 5);
		Vector VNew=new Vector (5, 5);
		JAC_FileUtil	FUtil;

		FUtil = new JAC_FileUtil (d_PovFile);
		if (!FUtil.exists())
			if (LPEOptions.s_libdir != null)
				FUtil = new JAC_FileUtil (LPEOptions.s_libdir, 
							  d_PovFile);

		if (JAC_GlobalDebug.debug() == 3)
			{
			System.out.println ("Range: " + xrange + 
					", " + yrange + 
					", " + zrange);
			System.out.println ("Center: " + xcenter + 
					", " + ycenter + 
					", " + zcenter);
			}

		// set the proper include file name 
		if (d_mode != LPERender.s_RENDER_MODE_MBLOB)
			{
			VOrg.addElement ("LSYSINCLUDEFILE");
			VNew.addElement (d_Incname);
			}

		// modify POV camera "looks at" statement 
		VOrg.addElement ("LSLOOKAT_X");
		VNew.addElement (String.valueOf(xcenter/xrange));
		VOrg.addElement ("LSLOOKAT_Y");
		VNew.addElement (String.valueOf(ycenter/yrange));
		VOrg.addElement ("LSLOOKAT_Z");
		VNew.addElement (String.valueOf(zcenter/zrange));

		// modify POV "scale" statement on the final object 
		VOrg.addElement ("LSSCALEX");
		VNew.addElement (String.valueOf(1/xrange));
		VOrg.addElement ("LSSCALEY");
		VNew.addElement (String.valueOf(1/yrange));
		VOrg.addElement ("LSSCALEZ");
		VNew.addElement (String.valueOf(1/zrange));
	
		// use stringReplace to create the new file
		FUtil.stringReplace (VOrg, VNew, d_Povname);
		return (0);
		}

	/** renderPovrayFile: issue the command to render the *.pov file */
	private int renderPovrayFile () throws InterruptedException, IOException
		{
		String	povcmd;
		Process	P;

		if (LPEOptions.s_renderBig)
			povcmd = new String ("povray +W800 +H600 -D +i " + d_Povname);
		else
			povcmd = new String ("povray +W200 +H150 -D +i " + d_Povname);

		if (d_Btn!=null)
			{
			d_Btn.setLabel ("Rendering ...");
			d_Btn.setVisible (true);
			}

		if (JAC_GlobalDebug.debug()  == 3)
			System.out.println ("Executing: " + povcmd); 

		P = Runtime.getRuntime().exec (povcmd);
		this.yield ();
		P.waitFor ();

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println("POVRay return code: "+ P.exitValue());

		return (P.exitValue());
		}

	/** convertTGAFile: issue the command to convert TGA file to JPG */
	private int convertTGAFile () throws InterruptedException, IOException
		{
		String	convertcmd = new String (
				"cjpeg -outfile " + d_Jpgname + " " + d_Tganame); 
		Process	P;

		if (d_Btn!=null)
			{
			d_Btn.setLabel ("Converting ...");
			d_Btn.setVisible (true);
			}

		if (JAC_GlobalDebug.debug()  == 3)
			System.out.println ("Executing: " + convertcmd); 

		P = Runtime.getRuntime().exec (convertcmd);
		this.yield ();
		P.waitFor ();

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println("Image conversion return code: "+ P.exitValue());

		return (P.exitValue());
		}

	
	/** copyPop1Jpg: copy the rendered jpg files to filenames 
	    consistent with the initial filename from the *.lsp file 
	    we read at startup
	**/
	private void copyPop1Jpg ()
		{
		int		lim = d_LPP.size();
		LParserObj 	tLPO;
		JAC_FileUtil	JFU;

		//System.out.println ("Trying to copy JPGs in " + d_Dirname);
		for (int i=0; i<lim; i++)
		    {
		    try { 
			tLPO = (LParserObj)d_LPP.elementAt(i);
			if (d_directory)
				JFU = new JAC_FileUtil (d_Dirname, 
					new String (tLPO.fullName() + ".jpg"));
			else
				JFU = new JAC_FileUtil (new String (tLPO.fullName() + ".jpg"));

			if (JFU.exists())
				JFU.copy (new String (JAC_FileUtil.baseName (tLPO.getName()) + ".jpg"));
			}
		    catch (Exception e) {
			System.out.println ("Caught Exception in LPEGui::copyPop1Jpg(): " + e);
			}
		    }
		}

}

