/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ***** LPEvol: Main class, stores and manipulates a LParser Population **** */
/* ************************************************************************** */
public class LPEvol 
{
	public static final float	s_version=(float)0.54;
	private LParserPop		d_LPP;
	private LPEOptions 		d_Opt;
	private boolean 		d_benchverbose=false;

	/** main: run the program */
	public static void main (String[] args)
		{
		LPEvol LPE=new LPEvol (args);
		}

	/** Constructor: init the progam, look for global flags, 
	    then run whatever necessary 
	**/
	public LPEvol (String[] args)
		{
		LPEGui			RLPGui=null; // runnable LParser Gui

		// JAC_GlobalRandom.init ();

		// process args we need right away (verbose, seed, mode, bench, ...) 
		for (int i=0; i<args.length; i++)
		    {
		    if (args[i].equals ("-v") || args[i].equals ("-verbose"))
			{
			JAC_GlobalDebug.verbose (true);
			System.out.println ("Caught -verbose option: verbose=" + JAC_GlobalDebug.verbose());
			}

		    else
		    if (args[i].equals ("-seed")) 
			{
			if (args.length > i)
				{
				if (JAC_GlobalVerbose.verbose())
					System.out.println ("Caught -seed option: seed=" + new Long (args[i+i]));
				JAC_GlobalRandom.init (new Long(args[++i]).longValue());
				}
			else
				{
				System.out.println ("ParseArgs() error: -seed not followed by seed");
				System.exit (0);
				}

			}

		    else
		    if (args[i].startsWith ("-dbg"))
			if (args[i].length() > 4)
				{
				if (Character.isDigit(args[i].charAt(4))) 
					{
					Integer	Level=new Integer (args[i].substring (4, 5));
					JAC_GlobalDebug.debug (Level.intValue());
					System.out.println ("Caught -dbg option: debug=" + JAC_GlobalDebug.debug());
					}
				else
					{
					System.out.println ("MParseArgs() error: -dbg not followed by digit ... ");
					System.exit (0);
					}
				}
			else
				{
				System.out.println ("MParseArgs() error: incomplete -dbg flag");
				System.exit (0);
				}

		    else
		    if (args[i].equals ("-bench"))
			{
			JAC_GlobalDebug.debug (0);
			JAC_GlobalRandom.init (0);
			worthlessBenchmark ();
			System.exit (0);
			}

		    else
		    if (args[i].equals ("-bbench"))
			{
			JAC_GlobalDebug.debug (0);
			JAC_GlobalRandom.init (0);
			longWorthlessBenchmark ();
			System.exit (0);
			}
		    }

		// Build an empty LParser Population 
		d_LPP = new LParserPop ();

		// process command line 
		d_Opt = new LPEOptions (d_LPP, args);

		// interactive gui || graphical batch 
		if (d_Opt.s_gui)
			{
			int	n;
			// Initialize the Gui, gui becomes main thread
			RLPGui = new LPEGui (d_LPP, "LPEvol");
			RLPGui.setName ("LPEGui Thread");
			if (d_LPP.size() > 0)
				{
				if (LPEOptions.s_recycleImages)
					RLPGui.recycleImages ();
				}
			RLPGui.start ();
/*
			try {
				RLPGui.join (); }
			catch (InterruptedException e) {
				System.out.println ("Caught exception :" 
					+ e + " in LPEvol ()");
			    }
*/
			}

		// (nongui) batch mode 
		if (d_Opt.s_batch)
			runBatch (RLPGui, true);
		}

	/** runBatch: run a batch job for maxgen generations */
	public void runBatch (LPEGui LPGui, boolean do_write)
		{
		int		i=0;
		String		Dir;
		LParserGA	LPGA=null;
		LParserPop	newLPP=null; 

		if (d_LPP.size() == 0)
			{
			System.out.println ("Sanity Check Failed: population is empty in runBatch() ... Exiting");
			LPEOptions.usage ();
			}

		if (d_benchverbose)
			System.out.print ("Generation: ");

		Dir = ((LParserObj)d_LPP.elementAt(0)).fullName();
		while (i < d_Opt.s_maxgen)
			{
			LParserObj		LPO = (LParserObj)d_LPP.elementAt(0);

			if (d_benchverbose)
				System.out.print (i + " ");

			LPGA=new LParserGA (LPO);
			LPGA.setChromOffset (LPEOptions.s_chromOffset);
			LPGA.setNumberBreed (LPEOptions.s_numberBreed, LPEOptions.s_numberBreedProb);
                        LPGA.setCrossBeforeNumber (LPEOptions.s_crossBeforeNumber);
                        LPGA.setLimitBrackets (LPEOptions.s_limitBrackets);
			LPGA.setAppend (LPEOptions.s_append, LPEOptions.s_appendReplace);

			// build population 
			newLPP = LPGA.breed(d_LPP, d_Opt.s_popsize);
			newLPP.setName (d_LPP.getName());
			d_LPP = newLPP;

			if (d_Opt.s_write || d_Opt.s_gui)
				d_LPP.write ();
			if (d_Opt.s_gui)
				{
				LPGui.setTitle (d_LPP.generationName());
				d_LPP.render (LPGui.getButtons(), true, true, true);
				}
			if (d_Opt.s_directory)
				{
				LPEFileUtil	LFU; 

				LFU = new LPEFileUtil (Dir, LPO.generationName()); 
				LFU.cleanupFiles();
				}
			i++;
			}

		if (do_write)
			d_LPP.write ();

		if (d_benchverbose)
			System.out.println ("");
		}

	/** worthlessBenchmark: not much of a benchmark but a good way 
	    to stress test the syntax code during development 
	*/
	public float worthlessBenchmark ()
		{
		Date 	DStart, DStop;
		int	rc; 
		long	mstime, norgs;
		float 	score=0;

		d_benchverbose = true;
		d_Opt.s_batch = true;
		d_Opt.s_write = false;
		d_Opt.s_gui = false;
		d_Opt.s_maxgen = 1000;
		norgs=d_Opt.s_maxgen*d_Opt.s_popsize;

		d_LPP = new LParserPop ();
		if ((rc=d_LPP.popFile("_samplepop.lsp")) < 0)
			{
			System.out.println ("Error reading population file: rc=" + rc + " ... Exiting");
			System.exit (rc);
			}

		DStart = new Date();
		System.out.println ("Benchmark (" + d_Opt.s_maxgen + ") started at " + DStart.getTime());
		runBatch (null, false);
		DStop = new Date();
		System.out.println ("Benchmark (" + d_Opt.s_maxgen + ") ended at " + DStart.getTime());

		mstime = DStop.getTime() - DStart.getTime();
		score = (float)norgs/(float)mstime;
		System.out.println ("Elapsed time (ms): " + mstime);
		System.out.println ("Number of organisms generated (" + d_Opt.s_maxgen + "x" + d_Opt.s_popsize + "): " + norgs);
		System.out.println ("Orgs per ms: " + score);

		return (score);
		}

	/** worthlessBenchmark: as above, just longer */
	public float longWorthlessBenchmark ()
		{
		Date 	DStart, DStop;
		long	mstime;
		float	t, score, sum=0, high=0, low=100000, avg;

		DStart = new Date();
		System.out.println ("Long Benchmark started at " + DStart.getTime());
		for (int i=0; i<10; i++)
			{
			t=worthlessBenchmark();
			sum+=t;
			if (t > high)
				high = t;
			if (t < low)
				low = t;
			}

		DStop = new Date();
		mstime = DStop.getTime() - DStart.getTime();
		score = sum/10;
		System.out.println ("Elapsed time (ms): " + mstime);
		System.out.println ("Worst score: " + low);
		System.out.println ("Best score: " + high);
		System.out.println ("Average score: " + score);
		return (score);
		}
}
