/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* *********************************************************************** */
/* *********** LParserIO: Class to handle the IO for a LParserObj ************** */
/* *********************************************************************** */
public class LParserIO 
{
	/** write: write a specific LPO */
	public static int write (LParserObj	LPO)
		{
		try {
			LParserWriter	Lsw = new LParserWriter (LPO);
			Lsw.write ();
			return (0);
			}
		catch (IOException e) {
			System.out.println ("LParserIO::write(): Caught exception " + e); 
			}
		return (-1);
		}

	/** Read: read a specific LPO */
	public static LParserObj read (LParserObj	LPO)
		{
		try {
			LParserReader		Lsr = new LParserReader (LPO);
			Lsr.read (); 
			return (LPO);
			}
		catch (IOException e) {
			System.out.println ("LParserIO:Read(): caught exception " + e); 
			}
		return (null);
		}
} 


/* *********************************************************************** */
/* ********** LParserWriter: Class to write 1 LParser Object ************* */
/* *********************************************************************** */
class LParserWriter extends JAC_FileWriter
{
	private	LParserObj		d_LPO = null;

	/** Constructor: read filename through Superclass and set name */
	public LParserWriter (LParserObj LPO) throws FileNotFoundException, IOException
		{
		super (new String (LPO.fullName()+".ls"));
		d_LPO = LPO;
		}

	/** write: write this org to file */
	public int write () throws IOException 
		{
		int		i, lim=d_LPO.size();
		Vector		VFinal = new Vector (lim+1, 5);
		String		Line=null;
		Date		D=new Date ();
		String		Name=d_LPO.fullName ();
		LParserObj	[]P=d_LPO.getParents();

		Name = new String (Name + ".ls");
		if (JAC_GlobalDebug.verbose())
			System.out.println ("Writing " + Name + " (" + lim + ")");

		D.getTime ();

		if (P[0] == null)
			{
			System.out.println ("Sanity Check failed in LParserWriter::write(): P[0]==null ... Exiting");
			System.exit (1);
			}

		Line = new String ("#\n# " + Name + 
				   "\n# Created by LParserEvol on " + 
				   D.toString() + "\n#\n# Parents: " + 
				   P[0].fullName() + " + " + 
				   P[1].fullName() + "\n#");
		VFinal.addElement (Line);

		for (i=0; i<LParserObj.s_NUM_VALUES; i++)
			{
			Line = new Float (d_LPO.getValue(i)).toString();
			VFinal.addElement (Line);
			}

		for (i=0; i<lim; i++)
			{
			Line = (String)d_LPO.elementAt(i);
			VFinal.addElement (Line);
			}
		Line = new String ("@\n");
		VFinal.addElement (Line);

		this.write (VFinal, true);
		this.close(); 
		return (0);
		}
}


/* *********************************************************************** */
/* ****** LParserReader: Class to read and and store 1 LParser File ****** */
/* *********************************************************************** */
class LParserReader extends JAC_FileReader 
{
	private LParserObj		d_LPO = null;

	/** LParserReader: constructor: call superclass constructor */
	public LParserReader (LParserObj LPO) throws FileNotFoundException 
		{
		super (LPO.getName(), true);
		d_LPO = LPO;
		}

	/** read: Read the rules from the *.ls file */
	public int read () throws IOException 
		{
		int	lim, i, ti=0;
		String	S = null;
		Vector  VList = null;

		VList = this.readList (0, "@", "#");

		// get limit, ignore closing '@'
		lim = VList.size();
		if (((String)VList.elementAt(lim-1)).startsWith ("@"))
			lim--;

		if (lim < 5)
			JAC_Bailout.bailout (new String (
				"Sanity Check failed in LParserReader::read(): only " + lim + " data lines ..." + 
				"\nIs is a LParser file (looks incomplete)? Exiting"), 
				1);

		for (i=0; i<LParserObj.s_NUM_VALUES; i++)
			{
			S = (String)VList.elementAt(i);
			d_LPO.setValue (i, new Float(S).floatValue());
			d_LPO.setParents (d_LPO, d_LPO);
			}

		for (i=LParserObj.s_NUM_VALUES; i<lim; i++)
			{
			S = (String)VList.elementAt(i);

			if (i>LParserObj.s_NUM_VALUES+1 && S.indexOf("=") == -1)
				JAC_Bailout.bailout (new String ("Sanity Check failed in LParserReader::read(): can't find '='" + 
					"\nLine: " + S + 
					"\nIs is a LParser file? Exiting"), 1);

			if ((ti=S.indexOf ("#")) != -1)
				S = new String (S.substring (0, ti-1));
			d_LPO.addElement (S);
			}
					
		this.close();
		return (0);
		}
}


