/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.util.*;


/* *********************************************************************** */
/* * LParserLangUtil: Class to implement lparser language util functions * */
/* *********************************************************************** */
public class LParserLangUtil 
{
	private String		d_Line=null;
	private boolean		d_crossBeforeNumber=true;
	//private final char 	PUSH_TOK=LParserGA.PUSH_TOK;	// push token (LParser syntax)
	//private final char 	POP_TOK=LParserGA.POP_TOK;	// pop token (LParser syntax)
	private final char 	PUSH_TOK='[';		// pop token (LParser syntax)
	private final char 	POP_TOK=']';		// pop token (LParser syntax)

	/** Constructor: init with d_crossBeforeNumber=true */
	public LParserLangUtil (String Line)
		{
		this (Line, true);
		}

	/** Constructor: check valid LParserObj and assign local LPO */
	public LParserLangUtil (String Line, boolean crossBeforeNumber)
		{
		if (Line == null)
			{
			System.err.println ("Error: Line==null in LParserLangUtil ... Exiting\n");
			System.exit (1);
			}
		d_Line = new String (Line);
		d_crossBeforeNumber = crossBeforeNumber;
		}


	/** fixNumberPoint: make sure n is a valid place to cut/crossover 
	    without clobbering a number. Returns the index of the first 
	    character before/after the deliming (round) brackets.
	**/
	public static int fixNumberPoint (String Line, int n, boolean cross_before)
		{
		int	lim = Line.length();

		if (n >= lim)
			JAC_Bailout.bailout (new String (
				"Sanity Check failed in LParserGa::fixNumberPoint() ... " + 
				"Index (" + n + ") out of range (max=" + lim + ") ... Exiting"), 
				-1);

		if (cross_before) 
			while (lsysNumber (Line.charAt(n)) && n>0)
				n--;
		else
			while (lsysNumber (Line.charAt(n)) && n<lim)
				n++;

		return (n);
		}

	/** ppBalance: default invocation, reduce imbalance to 0 */
	public String ppBalance ()
		{
		return (ppBalance(0, true));
		}

	/** ppBalance: return Line with it's PUSH_TOKs and POP_TOKs 
	    balanced. If the intial imbalance is greater than t_b, 
	    extra push/pop tokens are removed until the imbalance is 
	    reduced to t_b+1. Once this is done, the missing push/pop 
	    token is added to the line. 
	**/
	public String ppBalance (int target_balance, boolean insert) // PushPopBalance
		{
		int	bal=0, 
			lim=d_Line.length();
		String	NewLine=d_Line;

		if (d_Line.length() == 0)
			JAC_Bailout.bailout ("Sanity Check failed in LParserGa::ppBalance() ... can't balance empty Line");

		bal = JAC_StringUtil.stringBalance (d_Line, PUSH_TOK, POP_TOK);

		if (bal == 0)
			return (NewLine);

		if (JAC_GlobalDebug.debug() == 3)
			System.out.println ("LParserGa::ppBalance: bal=" + bal + ", " + d_Line);

		if (bal < 0)
			{
			
			if (insert)
				{
				String tLine = reduceImbalance (bal, target_balance+1, POP_TOK);
				NewLine = insertPush (tLine);
				}
			else
				NewLine = reduceImbalance (bal, target_balance, POP_TOK);
			}
		else
		if (bal > 0)
			{
			if (insert)
				{
				String tLine = reduceImbalance (bal, target_balance+1, PUSH_TOK);
				NewLine = insertPop (tLine);
				}
			else
				NewLine = reduceImbalance (bal, target_balance, PUSH_TOK);
			}

		return (NewLine);
		}

	/** reduceImbalance: reduce the effective (==absolute) balance (bal)
	    to desired by randomly removing tokens from d_Line
	**/
	public String reduceImbalance (int bal, int desired, char tok)
		{
		int	n=0, last=0, 
			bad_luck, bl_idx,	 // index index + object to be removed
			lim=d_Line.length();
		String	tLine;
		Vector	VIdx = new Vector (3, 3);

		bal = Math.abs(bal);
		desired = Math.abs(desired);

		if (desired > bal)
			{
			JAC_Bailout.bailout (new String (
				"Sanity Check failed in LParserLangUtil::reduceImbalance() ... " 
				+ desired + " > " + bal), -1);;
			System.exit (1);
			}
		else
		if (desired == bal)
			return (d_Line);

		if (JAC_GlobalDebug.debug() > 0)
			System.out.println ("LParserGa::reduceImbalance: reducing " + bal + " to " + desired);

		// build a Vector of tok array subscripts
		while ((n=d_Line.substring(last, lim).indexOf(tok)) >= 0)
			{
			VIdx.addElement (new Integer (last+n));
			last=last+n+1;
			}

		tLine = d_Line;
		while (bal > desired)
			{
			int 	lim2=VIdx.size();

			// get a random VIdx subscript
			bad_luck = (int)(JAC_GlobalRandom.nextFloat()*lim2);
			if (bad_luck == lim2 && lim2 > 0)
				bad_luck--;

			// get the String subscript bad_luck points to 
			//System.out.println ("reduceImbalance: tok=" + tok + ", bal=" + bal + ", size=" + lim2 + ", " + d_Line);
			bl_idx = ((Integer)VIdx.elementAt(bad_luck)).intValue();
			//System.out.println (tLine + "   bl_idx=" + bl_idx);
			//System.out.println ("removing " + bl_idx + "	" + tLine.charAt(bl_idx) + "   "  + tLine );

			// remove the bad_luck 'tok'
			if (bl_idx == 0)
				tLine = new String (tLine.substring (1, tLine.length()));
			else
			if (bl_idx == tLine.length())
				tLine = new String (tLine.substring (0, tLine.length()-1));
			else
				tLine = new String (tLine.substring(0,bl_idx) + 
					tLine.substring(bl_idx+1,tLine.length()));

			// remove index entry and update counters;
			VIdx.removeElementAt (bad_luck);
			lim--;
			lim2--;
			bal--;

			// line is shorter now: update the tok index Vector 
			for (int i=0; i<lim2; i++)
				{
				int bl_idx2;

				bl_idx2 = ((Integer)VIdx.elementAt(i)).intValue();
				if (bl_idx2 >= bl_idx)
					VIdx.setElementAt (new Integer (bl_idx2-1), i);
				}
			}

		if (JAC_GlobalDebug.debug() == 3)
			System.out.println ("LParserGa::reduceImbalance: " + tLine);

		return (tLine);
		}

	/** insertPush : return the difference in the number of push or pop 
	    ([]) tokens in Line 
	**/
	private String insertPush (String Line)
		{
		int 	tpop, 
			tpush=0,
			insert,
			last=0,
			lim=Line.length();
		String 	NewLine;

		// determine the line segment on which to operate 
		while ((tpop=Line.substring(last,lim).indexOf(POP_TOK)) != -1)
			{
			tpush=Line.substring(last,tpop+last).indexOf(PUSH_TOK);
			//System.out.println (Line + " Xlast=" + last + " tpush=" + tpush + " tpop=" + tpop);
			if (tpush == -1)
				break;
			last=tpop+last+1;
			}

		// determine the place to insert 
		insert = (int)(JAC_GlobalRandom.nextFloat()*(tpop+last));
		if (insert == tpop && insert > 0)
			insert--;
		insert = fixNumberPoint (Line, insert, d_crossBeforeNumber);

		// insert PUSH_TOK
		NewLine = new String (Line.substring(0,insert) + PUSH_TOK + 
					Line.substring(insert,lim));

		if (!NewLine.equals(Line) && JAC_GlobalDebug.debug() == 3)
			System.out.println ("LParserLangUtil::insertPush: \n" + Line + 
				" insert " + PUSH_TOK + " at " + 
				insert + "\n" + NewLine);

		return (NewLine);
		}

	/** insertPop: return the difference in the number of push or pop 
	    ([]) tokens in Line 
	**/
	private String insertPop (String Line)
		{
		int 	tpush, 
			tpop=0,
			insert,
			last=0,
			lim=Line.length();
		String 	NewLine;

		// determine the line segment on which to operate 
		while ((tpush=Line.substring(last,lim).indexOf(PUSH_TOK)) != -1)
			{
			// bailout if we have "ab[cd[ef]gh"
			if (tpush < tpop)
				break;
			tpush+=last;
			//System.out.println (Line + " Xlast=" + last + " tpush=" + tpush + " tpop=" + tpop);
			tpop=Line.substring(tpush+1,lim).indexOf(POP_TOK);
			if (tpop == -1) 
				break;
			last=tpush+last+1; 
			if (last > Line.length())
				break;
			}

		// determine the place to insert 
		insert = (int)(JAC_GlobalRandom.nextFloat()*(lim-(tpush))+(tpush));
		if (insert == lim)
			tpush--;
		insert = fixNumberPoint (Line, insert, d_crossBeforeNumber);
		if ((Line.charAt(insert)==PUSH_TOK && insert<lim-1) ||
		    insert==tpush)
			insert++;

		// insert POP_TOK
		NewLine = new String (Line.substring(0,insert) + POP_TOK + 
					Line.substring(insert,lim));

		if (!NewLine.equals(Line) && JAC_GlobalDebug.debug() == 3)
			System.out.println ("LParserLangUtil::insertPop: \n" + Line + 
				" insert " + POP_TOK + " at " + 
				insert + "\n" + NewLine);

		return (NewLine);
		}

	/** lsysNumber: return true if c is in a LParser number */
	private static boolean lsysNumber (char c)
		{
		if (c=='.' || c=='(' || c==')' || Character.isDigit(c)) 
			return (true);
		else
			return (false);
		}
}

