/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* *********************************************************************** */
/* ***** LParserObj: Class to extend and manipulate 1 LParser object ***** */
/* *********************************************************************** */
public class LParserObj extends LParserCore
{
	private	static int	s_generation=0;		// generation count
	private	static int	s_nchild=0;		// number of children from this generation 
	private int		d_nchild=0;		// this orgs child number 
	private boolean		d_selected=false;	// flag for GA selection
	private boolean		d_directory=LPEOptions.s_directory;	// flag for directory use 
	private LParserObj	d_Parent1, d_Parent2;	// parents

	/** Constructor: init superclass and initialize selection status */
	public LParserObj () 
		{
		super ();
		d_selected=false;
		}

	/** Constructor: read Fname and register name */
	public LParserObj (String Fname) throws FileNotFoundException 
		{
		this ();
		this.d_Name = new String (Fname);
		LParserIO.read (this);
		d_selected=false;
		}

	/** copy: copy LPO over 'this' */
	public int copy	(LParserObj LPO)
		{
		if (LPO == null)
			return (-1);

		int		lim=LPO.size();
		String		word;
		StringTokenizer	sTok;

		this.setSize(0);
		for (int i=0; i<lim; i++)
			this.addElement ((String)LPO.elementAt(i));
		this.setName (LPO.getName());
		d_selected = true;

		// set parents 
		LParserObj	[]P = LPO.getParents();
		d_Parent1 = P[0];
		d_Parent2 = P[1];

		return (0);
		}

	/** Constructor: read Fname and register name */
	public void write () throws FileNotFoundException 
		{
		LParserIO.write (this);
		}

	/** setSelected: set org selection status */
	public void setSelected (boolean selected)
		{
		d_selected=selected;
		if (JAC_GlobalDebug.debug() > 0) 
			System.out.println (this.d_Name + ": " + "d_selected = " + d_selected);
		}

	/** getSelected: return selection status */
	public boolean getSelected ()
		{
		return (d_selected);
		}

	/** generationName: add generations to Name() */ 
	public String generationName ()
		{
		String          Base, Fname;

		Base = JAC_FileUtil.baseName (this.d_Name);
		Fname = new String (Base + "-" + s_generation);

		return (Fname);
		}

	/** fullName: transform Name() into something which is more 
	    meaningful when dealing with multiple generations/organisms
	**/ 
	public String fullName ()
		{
		return (new String (generationName() + "-" + d_nchild));
		}

	/** getGeneration: return the generation count */
	public int getGeneration ()
		{
		return (s_generation);
		}

	/** setGeneration: assign and return the new generation count */
	private int setGeneration (int generation)
		{
		s_nchild = 0;
		s_generation = generation;
		return (s_generation);
		}

	/** nextGeneration: up generation count and reset number of children */
	public int nextGeneration ()
		{
		s_nchild = 0;
		return (++s_generation);
		}

	/** children: return the number of children in this generation */
	public int childCount ()
		{
		return (d_nchild);
		}

	/** nextChild: up child count */
	public int nextChild ()
		{
		d_nchild = s_nchild++;
		return (d_nchild);
		}

	/** setParents: assign the parents */
	public void setParents (LParserObj Parent1, LParserObj Parent2)
		{
		d_Parent1 = Parent1;
		d_Parent2 = Parent2;
		}

	/** getParentSubscripts: return the concatenated parent names */
	public LParserObj[] getParents ()
		{
		LParserObj	[]P=new LParserObj[2];

		P[0] = d_Parent1;
		P[1] = d_Parent2;
		return (P);
		}

}


/* *********************************************************************** */
/* * LParserCore: Class to store LParser org + provide some util methods * */
/* *********************************************************************** */
abstract class LParserCore extends Vector 
{
	public static int		s_NUM_VALUES=3;

	protected String		d_Name;
	protected float[]		d_values;

	/** LParserCore: constructor: call superclass constructor */
	public LParserCore ()
		{
		super (5, 5);
		d_Name = null;
		d_values = new float [s_NUM_VALUES];
		}

	/** print: Print the LParser rules */
	public void print ()
		{
		int	lim=this.size();

		System.out.println ("\nLParserCore: Printing " + d_Name);
		for (int i=0; i<lim; i++)
			System.out.println (i + ": " + 
				(String)this.elementAt (i));
		}

	/** getValue: return the value at index n */
	public float getValue (int n)
		{
		return (d_values[n]);
		}

	/** setValue: set the value at index n to v */
	public void setValue (int n, float v)
		{
		d_values[n]=v;
		}

	/** getName: return current name */
	public String getName ()
		{
		return (d_Name);
		}

	/** setName: set org's name */
	public void setName (String Name)
		{
		d_Name = new String (Name);
		}
}
