/* ************************************************************************* *
   LPEvol - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* *********************************************************************** */
/* **** LParserPop: Vector Class to store a population of LParserObjs **** */
/* *********************************************************************** */
class LParserPop extends Vector
{
	public String		d_Name = new String ("Unnamed");

	/** Constructor: init superclass with appropriate defaults */
	public LParserPop ()
		{
		super (10, 10);
		}

	/** Constructor: init superclass and read population file Fname */
	public LParserPop (String Fname)
		{
		super (10, 10);
		popFile (Fname);
		}

	/** populationFile: read a population file's elements */
	public int popFile (String Fname)
		{
		Vector			VFNames;
		JAC_FileReader		Jfr;
		LParserObj		LPO;

		try {
			Jfr = new JAC_FileReader (Fname, true);
			d_Name = Fname;
			if (! (new File (Fname).exists()))
				return (-1);
			VFNames = Jfr.readList (0, "@", "#");
			if (VFNames.size() > 0)
				add (VFNames);
			else
				{
				System.out.println ("Error: " + Fname + " is empty ... Exiting");
				System.exit (1);
				}
			//Jfr.close();
			return (this.size());
			} 
		catch (IOException e) {
			System.out.println ("Caught exception in LParserPop::popFile(): " + Fname + " " + e);
			}
		return (-1);
		}

	/** add: read and add the files in VFiles (vector of Strings) */
	public int add (Vector VFiles)
		{
		int	lim=VFiles.size();

		for (int i=0; i<lim; i++)
			{
			// System.out.println ("[ " + (String)(VFiles.elementAt(i)) + " ]");
			add ((String)(VFiles.elementAt(i)));
			}

		return (0);
		}

	/** add: read and add the organism in fname */
	public int add (String fname)
		{
		LParserObj			LPO;

		try {
			LPO = new LParserObj (fname);
			LPO.nextChild ();
			
			this.addElement (LPO);
			}
		catch (FileNotFoundException e)
			{
			System.out.println ("Caught exception in LParserPop::add(fname): Caught exception " + e);
			return (-1);
			}

		return (0);
		}

	/** add: add the organism LPO */
	public int add (LParserObj LPO)
		{
		if (LPO == null)
			return (-1);

		this.addElement (LPO);
		return (0);
		}

	/** setName: set the population name */
	public void setName (String Name)
		{
		d_Name = Name;
		}

	/** getName: return the population Name */
	public String getName ()
		{
		return (d_Name);
		}

	/** print: print all the organisms in the population */
	public void print ()
		{
		int		lim=this.size();
		LParserObj		LPO;

		if (JAC_GlobalDebug.verbose())
			System.out.println ("\nPrinting Population (" + lim + ") ...");
		for (int i=0; i<lim; i++)
			{
			LPO=(LParserObj)this.elementAt (i);
			LPO.print ();
			}
		}

	/** generationName: create a generation Name for this popultion */
	public String generationName ()
		{
		return (new String (JAC_FileUtil.baseName(d_Name) + "-" + 
			((LParserObj)this.elementAt(0)).getGeneration()));
		}

	/** write: write all the organisms in the population + write a 
	    a *.lsp population file listing each org name 
	**/
	public int write ()
		{
		int		lim=this.size();
		LParserObj		LPO;
		String		Line; 
		String		FName = null;
		Date		D=new Date ();
		Vector		VPopFile = new Vector (lim, lim); 
		JAC_FileWriter	PWriter;	// ^ Population Names file 

		if (JAC_GlobalDebug.verbose())
			System.out.println ("Saving Population (" + lim + ") ...");

		if (lim == 0)
			return (1);

		try {
		    // assign and write all the organisms
		    for (int i=0; i<lim; i++)
			{
			LPO=(LParserObj)this.elementAt (i);
			LPO.write ();
			if (i == 0)
				FName = new String (generationName() + ".lsp");

			// register org name 
			VPopFile.addElement (new String (LPO.fullName() + ".ls\n"));
			}

		     if (VPopFile.size() > 0)
			{
		     	// build and insert comment segment for population file
		     	Line = new String ("#\n# " + FName + 
				"\n# Created by LPEvol on " + D.toString() + 
				"\n#\n# Population consists of " + lim + " orgs\n#\n");
		     	VPopFile.insertElementAt (Line, 0);

		     	PWriter = new JAC_FileWriter (FName);
		     	PWriter.write (VPopFile);
		     	PWriter.close ();
			return (0);
			}
		    return (1);
		    }
		catch (IOException e) {
		   System.out.println ("Caught exception in LParserPop::write():" + e);
		   }

		return (-1);
		}

	/** render: render all the organisms, don't wait */
	public int render ()
		{
		return (render (null, false, true, true));
		}

	/** render: render all the organisms onto VButton, don't wait,  */
	public int render (Vector VButton)
		{
		return (render (VButton, false, true, true));
		}

	/** Add: cd to the target directory for this population, then render 
	    all the organisms in the population, then cd back to the root dir.
	    if renderAll==False, we only render buttons which don't have an 
	    image yet.
	**/
	public int render (Vector VButton, boolean wait, 
			   boolean directory, boolean renderAll)
		{
		String			Cwd,
					Nwd;
		Properties		P;
		File			F;
		LPERender		LPR;
		LParserObj		LPO;

		// change directory to new dir 
		LPO = (LParserObj)this.elementAt(0);
		Cwd = JAC_FileUtil.getPwd ();
		Nwd = this.generationName();

		if (directory)
			{
			F = new File (Nwd);
			if (!F.exists())
				F.mkdir ();
			}

		// change to population directory
		// doesn't work due to java bug!
		// JAC_FileUtil.setPwd (Nwd);
	
		// render	
		LPR = new LPERender (this, VButton);
		LPR.setRenderAll (renderAll);
		LPR.start ();
		if (wait)
			{
			try {
				LPR.join (); }
			catch (InterruptedException e) {
				 System.out.println ("Caught exception :" 
					+ e + " in LPEvol ()"); 
			    }
			}

		// change back to original directory 
		// doesn't work due to java bug!
		// JAC_FileUtil.setPwd (Cwd);

		return (0);
		}
}


