/* ************************************************************************* *
   PovUtil - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	         http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ***** PovMaxVector_blob: parse a PovRay blob file for max vectors ******** */
/* ************************************************************************** */
public class PovMaxVector_blob extends PovMaxVector 
{
	/** main: for standalone use/testing */
	public static void main (String[] args)
                {
		PovMaxVector_blob	P=null;

		try {
			if (args.length == 1)
				{
                		P = new PovMaxVector_blob (args[0]); 
				P.run ();
				}
			else
				{
				System.err.println ("Usage: PovRescale <povray_file>");
				System.exit (1);
				}

			P.print ();
			}
		catch (FileNotFoundException e) {
			System.err.println ("Caught FileNotFoundException in PovMaxVector::main()"); }
		catch (IOException e) {
			System.err.println ("Caught IOException ... PovMaxVector::main()"); }
                }

	/** constructor: open file, establish keys and file max key vectors */
	public PovMaxVector_blob (String Fname) throws FileNotFoundException, 
						      IOException
		{
		super (Fname, 1);
		addKey (" ");
		}

	/** parse: read d_Bfr and process it */
	public void parse ()
		{
		int 		c=0, kstart,
				lim=this.size();
		String		Line,
				ElmLine;
		Double[]	D=new Double[3];
		PovVector 	Pv;
		Vector		VDouble;

		if (lim == 0)
			return;

		Pv=(PovVector)this.elementAt (0);

		try {
		    while ((Line=d_Bfr.readLine()) != null)
			{
			// check for each vector type if we can match it 
			this.d_lnumber++;
			if (Line.startsWith("component") &&
			    (kstart=Line.indexOf("<")) != -1)
				{
				ElmLine = Line.substring(Line.substring(0, kstart-2).lastIndexOf(" "));
				VDouble=parse_vector (ElmLine);

				if (VDouble == null)
					return;

				D[0]=(Double)VDouble.elementAt(0);
				D[1]=(Double)VDouble.elementAt(1);
				D[2]=(Double)VDouble.elementAt(2);
				d_PVMin.keepSmaller(D[0].doubleValue(), D[1].doubleValue(), D[2].doubleValue());
				d_PVMax.keepBigger(D[0].doubleValue(), D[1].doubleValue(), D[2].doubleValue());
				}

			if (++c%d_dotinterval == 0)
				{
				System.out.print (".");
				}
			this.yield ();
			}
		System.out.println ("");
		     }
		catch (IOException e)
			{
			}
		}
}


