/* ************************************************************************* *
   PovUtil - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	         http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ******* PovMaxVector_inc: parse a PovRay blob file for max vectors ******* */
/* ************************************************************************** */
public class PovMaxVector_inc extends PovMaxVector 
{
	/** main: for standalone use/testing */
	public static void main (String[] args)
                {
		PovMaxVector_inc 	P=null;

		try {
			if (args.length == 1)
				{
                		P = new PovMaxVector_inc (args[0]); 
				P.run ();
				}
			else
				{
				System.err.println ("Usage: PovRescale <povray_file>");
				System.exit (1);
				}

			P.print ();
			}
		catch (FileNotFoundException e) {
			System.err.println ("Caught FileNotFoundException in PovMaxVector::main()"); }
		catch (IOException e) {
			System.err.println ("Caught IOException ... PovMaxVector::main()"); }
                }

	/** constructor: open file, establish keys and file max key vectors */
	public PovMaxVector_inc (String Fname) throws FileNotFoundException, 
						      IOException
		{
		super (Fname);
		addKey ("translate");
		addKey ("triangle");
		addKey ("scale");
		}

	/** parse: read d_Bfr and process it */
	public void parse ()
		{
		int 		c=0, kstart,
				lim=this.size();
		String		Key,
				Line,
				ElmLine;
		Double[]	D=new Double[3];
		PovVector[] 	Pv;
		Vector		VDouble;

		if (lim == 0)
			return;

		// lookup table should be slightly faster
		Pv = new PovVector[lim];
		for (int i=0; i<lim; i++)
			Pv[i]=(PovVector)this.elementAt (i);

		try {
		    while ((Line=d_Bfr.readLine()) != null)
			{
			// check for each vector type if we can match it 
			this.d_lnumber++;
			for (int i=0; i<lim; i++)
				{
				Key=Pv[i].getKey();

				if ((kstart=Line.indexOf(Key)) != -1)
					{
					ElmLine = Line.substring (kstart, Line.length());
					VDouble=parse_vector (ElmLine);

					if (VDouble == null)
						return;

					int	flim = VDouble.size();
					if (flim%3 != 0)
						{
						System.err.println ("Degenerate Triangle on line " + c);
						}
					for (int j=0; j<flim; j+=3)
						{
						D[0]=(Double)VDouble.elementAt(j);
						D[1]=(Double)VDouble.elementAt(j+1);
						D[2]=(Double)VDouble.elementAt(j+2);
						d_PVMin.keepSmaller(D[0].doubleValue(), D[1].doubleValue(), D[2].doubleValue());
						d_PVMax.keepBigger(D[0].doubleValue(), D[1].doubleValue(), D[2].doubleValue());
						}
					}
				}

			if (++c%d_dotinterval == 0 && JAC_GlobalDebug.debug()>0)
				{
				System.out.print (".");
				}
			this.yield ();
			}
		if (JAC_GlobalDebug.debug()>0)
			System.out.println ("");
		     }
		catch (IOException e)
			{
			}
		}
}


