/* ************************************************************************* *
   PovUtil - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	         http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ***** PovMultiplyColors: provide a class which will take a color ********* */
/* ***** (string) and replace it with n shades (color_1 - color_n)  ********* */
/* ************************************************************************** */
public class PovMultiplyColors extends Vector 
{
	private int			d_maxshades=10;
	private int			d_dotinterval=200;
	private JAC_FileReader		d_inFile=null;
	private JAC_FileWriter		d_outFile=null;

	/** main: for standalone use/testing */
	public static void main (String[] args)
                {
		PovMultiplyColors	PMC=null;

		if (args.length < 3)
			{
			System.out.println ("Usage: PovMultiplyColors <infilename> <number_of_shades> <color1> ... <colorn>");
			System.exit (1);
			}

		try {
			int	argoff = 0;

			if (args[1].charAt(0) == '+')
				{
				argoff=1;
				PMC = new PovMultiplyColors (args[0], args[1].substring (1), 
						new Integer(args[1+argoff]).intValue());
				}
			else
				PMC = new PovMultiplyColors (args[0], new Integer(args[1]).intValue());

			for (int i=2+argoff; i<args.length; i++)
				PMC.addElement (args[i]);
		    }
		catch (Exception e)
			{ System.out.println ("Caught Exception in PovMultiplyColors::main(): "+ e + "\nBailing out ..."); 
			  System.exit (1);
			}

		PMC.run ();
                }

	/** constructor: open file, establish keys and file max key vectors */
	public PovMultiplyColors (String FInName, int maxshades) throws 
							FileNotFoundException, 
							IOException
		{
		this (FInName, null, maxshades);
		}

	/** constructor: open file, establish keys and file max key vectors */
	public PovMultiplyColors (String FInName, String FOutName, 
				  int maxshades) throws FileNotFoundException, 
							IOException
		{
		super (5, 5);
		d_inFile = new JAC_FileReader (FInName);
		if (FOutName != null)
			d_outFile = new JAC_FileWriter (FOutName);
		d_maxshades = maxshades;
		}

	/** run: parse and replace the file */
	public void run ()
		{
		if (this.size() == 0)
			{
			System.err.println ("Sanity Check Failed in PovMultiplyColors::run(): size==0 ... Exiting\n");
			System.exit (1);
			}

		parseReplace (); 

		try { d_inFile.close(); }
		catch (IOException e) {
			System.out.println ("Caught IOException in PovMuliplyColors::run() while closing file ...");
			}
		
		}

	/** parse: read d_inFile and process it */
	public void parseReplace ()
		{
		int 		loc, j, c=0, kstart,
				lim=this.size();
		String		Key,
				Line,
				FLine,
				ElmLine;
		Double[]	D=new Double[3];
		String[]	Pc;
		Vector		VDouble;

		if (lim == 0)
			return;

		// lookup table should be slightly faster
		Pc = new String[lim];
		for (int i=0; i<lim; i++)
			Pc[i]=(String)this.elementAt (i);

		try {
		    while ((Line=d_inFile.readLine()) != null)
			{
			FLine=new String ();
			// check for each vector type if we can match it 
			if ((loc=Line.indexOf ("color")) != -1)
			    {
			    StringTokenizer	STok = new StringTokenizer (Line.substring(loc), " {}", true);
			    String		tok;

			    FLine = Line.substring (0, loc);

			    // parse rest of line
			    while (STok.hasMoreTokens())
				{
				tok = STok.nextToken();
			        for (j=0; j<lim; j++)
				    {
				    if (tok.equals (Pc[j]))
					{
					FLine = FLine.concat (rndColorName (Pc[j]));
					j = lim+1;
					}
				    }

				if (j == lim)
				    FLine = FLine.concat (tok);
				}
			    }
			else
				FLine = Line;

			if (d_outFile != null)
				{
				d_outFile.write (FLine, 0, FLine.length());
				d_outFile.newLine ();
				}
			else
				System.out.println (FLine);

			if (++c%d_dotinterval == 0 && 
			    JAC_GlobalDebug.verbose() &&
			    d_outFile != null)
					System.out.print (".");
			}

		     d_outFile.close();
		     if (JAC_GlobalDebug.verbose() && d_outFile!=null)
		     		System.out.println ("");
		     }
		catch (IOException e)
			{
			}
		}

	public String rndColorName (String Color)
		{
		int	r = (int)(JAC_GlobalRandom.nextFloat()*d_maxshades+1);

		if (r == d_maxshades+1)
			r = d_maxshades;

		return (new String (Color + "_" + Integer.toString(r)));
		}

	/** addKey: establish a new Key */
	public String addColor (String Key)
		{
		String		S=null;

		if (d_maxshades > 0)
			if (this.size() >= d_maxshades)
				{
				System.err.println ("Warning: PovMaxVector::addColor(): cancelled to prevent overflow");
				return (null);
				}

		S = Key;
		this.addElement (S);
		return (S);
		}

}


