/* ************************************************************************* *
   PovUtil - (C) Copyright 1998 Robert Gasch (Robert_Gasch@peoplesoft.com)
	        http://www.peoplesoft.com/peoplepages/g/robert_gasch/index.htm

   Permission to use, copy, modify and distribute this software for any 
   purpose and without fee is hereby granted, provided that this copyright
   notice appear in all copies as well as supporting documentation. All
   work developed as a consequence of the use of this program should duly
   acknowledge such use.

   No representations are made about the suitability of this software for
   any purpose. This software is provided "as is" without express or implied 
   warranty.

   See the GNU General Public Licence for more information.
 * ************************************************************************* */


import java.lang.*;
import java.io.*;
import java.util.*;


/* ************************************************************************** */
/* ******* PovVector: Class to keep track of 1 specific PovVector *********** */
/* ************************************************************************** */
public class PovVector
{
	private String 		d_Key;		// POV vector name 
	private double[]	d_vector;	// XYZ values for POV Vector

	/** constructor: establish new key with given values */
	public PovVector (String Key)
		{
		d_vector = new double[3];
		if (Key == null)
			bailout ("Sanity Check Failed: Key==null in PovVector::PovVector ... Exiting");
		else if (Key.length() == 0)
			bailout ("Sanity Check Failed: Key.length()==0 in PovVector::PovVector ... Exiting");
		d_Key=Key;
		d_vector[0]=0;
		d_vector[1]=0;
		d_vector[2]=0;
		}

	public void setValues (double x, double y, double z)
		{
		d_vector[0]=x;
		d_vector[1]=y;
		d_vector[2]=z;
		}

	/** print: print the properties of this PovVector */
	public void print ()
		{
		System.out.println ("PovVector: " + d_Key + ": " + d_vector[0] + 
			", " + d_vector[1] + ", " + d_vector[2]);
		}

	/** Name: return the name of this PMV */
	public String getKey ()
		{
		return (d_Key);
		}

	/** Values: return the 3 values for XYZ */
	public double[] getValues ()
		{
		return (d_vector);
		}

	/** keepBigger: updates d_vector if for any parameter which is a new max */
	public void keepBigger (double maxx, double maxy, double maxz)
		{
		if (maxx > d_vector[0])
			d_vector[0] = maxx;
		if (maxy > d_vector[1])
			d_vector[1] = maxy;
		if (maxz > d_vector[2])
			d_vector[2] = maxz;
		}

	/** keepBigger: updates d_vector if for any parameter which is a new max */
	public void keepSmaller (double minx, double miny, double minz)
		{
		if (minx < d_vector[0])
			d_vector[0] = minx;
		if (miny < d_vector[1])
			d_vector[1] = miny;
		if (minz < d_vector[2])
			d_vector[2] = minz;
		}

	/** bailout: print error message and exit */
	private void bailout (String ErrMsg)
		{
		System.err.println (ErrMsg);
		System.exit (1);
		}
}


